/**
 ****************************************************************************
 * @file 	D_Driver.c
 * @brief	Motor vector control driver Source File
 * @version	V2.1
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>
#include "ipdefine.h"
#include "mcuip_drv.h"
#include "E_Sub.h"

#define DEFINE_APP
#include "D_Driver.h"
#undef	DEFINE_APP

/*===================================================================*
	  Local Variable Definition
 *===================================================================*/
/*===================================================================*
	  Local Proto Type Definition
 *===================================================================*/
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
static int32_t Limit_sub(int64_t _indata, int32_t _limlo, int32_t _limup);
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */


/**
 *********************************************************************************************
 * @brief		H_Encoder
 *				Adjust Z-phase, 0deg of electrical-angle
 * @param		vector_t* const			_motor	The structure address for motor vector control.
 * @param		TSB_EN_TypeDef * const	ENx		Select the encoder channel.
 *
 * @return		none
 *********************************************************************************************
 */
#if defined(__USE_ENCODER_CH0) || defined(__USE_ENCODER_CH1) || defined(__USE_ENCODER_CH2)
void H_Encoder(vector_t* const _motor, TSB_EN_TypeDef* const ENx)
{
	ENC_FlagStatus	EncFlg;

	EncFlg = ENC_GetENCFlag(ENx);

	/* Phase Z is detected? */
	if (EncFlg.Bit.ZPhaseDetectFlag == SET)
	{
		/* Get Encoder counter value */
		_motor->drv.EnCnt = ENC_GetCounterValue(ENx);
		_motor->drv.Z_detect = SET;
	}

	/* Calculation theta(electrical-angle) from encoder counter */
	_motor->drv.theta_enc = (uint32_t)(((uint64_t)_motor->drv.EnCnt * _motor->para.enc.pls2theta) + _motor->para.enc.deg_adjust);

	if (EncFlg.Bit.RotationDirection == 1)
	{
		/* CW */
		if (_motor->drv.EnCnt >= _motor->drv.EnCnt1)
		{
//			_motor->drv.EnCnt_dev = (q15_t)(_motor->drv.EnCnt - _motor->drv.EnCnt1);
			_motor->drv.EnCnt_dev = (int32_t)(_motor->drv.EnCnt - _motor->drv.EnCnt1);
		}
		else
		{
//			_motor->drv.EnCnt_dev = (q15_t)(_motor->para.enc.plsnum + _motor->drv.EnCnt - _motor->drv.EnCnt1);
			_motor->drv.EnCnt_dev = (int32_t)(_motor->para.enc.plsnum + _motor->drv.EnCnt - _motor->drv.EnCnt1);
		}
	}
	else
	{
		/* CCW */
		if (_motor->drv.EnCnt1 >= _motor->drv.EnCnt)
		{
//			_motor->drv.EnCnt_dev = (q15_t)((_motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
			_motor->drv.EnCnt_dev = (int32_t)((_motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
		}
		else
		{
//			_motor->drv.EnCnt_dev = (q15_t)((_motor->para.enc.plsnum + _motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
			_motor->drv.EnCnt_dev = (int32_t)((_motor->para.enc.plsnum + _motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
		}
	}

	_motor->drv.EnCnt1 = _motor->drv.EnCnt;

	/* Cal omega(electrical-angle) from encoder counter deviation */
	_motor->drv.omega_enc_raw = (q15_t)(_motor->drv.EnCnt_dev * _motor->para.enc.pls2omega);
	_motor->drv.omega_enc_ave.word = calc_ave(_motor->drv.omega_enc_ave.word, _motor->drv.omega_enc_raw << 16 , cOMEGAENC_AVE);
	_motor->drv.omega_enc = _motor->drv.omega_enc_ave.half[1];

	if (_motor->drv.command.encoder == SET)
	{
		_motor->drv.omega.word = _motor->drv.omega_enc << 16;

		if (_motor->drv.vector_cmd.F_vcomm_theta == SET)
		{
			if (_motor->drv.vector_cmd.F_vcomm_omega == SET)
			{
				if (EncFlg.Bit.ZPhaseDetectFlag == 1)
				{
					/* Phase Z is detected */
					_motor->drv.theta.word = _motor->drv.theta_enc;
				}
				else
				{
					/* THETAn = THETAn-1 + OMEGA * t */
					/* theta = (omega * 2^16 * HZ_MAX) * Control period */
					_motor->drv.theta.word += ((_motor->drv.omega_com.half[1] * _motor->para.enc.ctrlprd) << 1);
				}
			}
			else
			{
				_motor->drv.theta.word += (_motor->drv.omega_com.half[1] * _motor->para.enc.ctrlprd) << 1;
			}
		}
		else
		{
			if (EncFlg.Bit.ZPhaseDetectFlag == 1)
			{
				/* Phase Z is detected */
				_motor->drv.theta.word = _motor->drv.theta_enc;
			}
			else
			{
				_motor->drv.theta.word = (_motor->drv.theta_com << 16);
			}
		}
	}
}
#endif /* __USE_ENCODER */

/**
 *********************************************************************************************
 * @brief		H_Hall_Encoder
 *				Adjust Z-phase, 0deg of electrical-angle
 * @param		vector_t* const			_motor	The structure address for motor vector control.
 * @param		TSB_EN_TypeDef * const	ENx		Select the encoder channel.
 *
 * @return		none
 *********************************************************************************************
 */
#if (defined(__USE_HALL_CH1) && defined(__USE_ENCODER_CH1))
void H_Hall_Encoder(vector_t* const _motor, TSB_EN_TypeDef* const ENx)
{
	ENC_FlagStatus	EncFlg;

	EncFlg = ENC_GetENCFlag(ENx);

	/* Get Z Phaze state. */
	_motor->HallCon->SensorFlg.enc_detect = EncFlg.Bit.ZPhaseDetectFlag;
	/* Phase Z is detected? */
	if (EncFlg.Bit.ZPhaseDetectFlag == SET)
	{
		/* Get Encoder counter value */
		_motor->drv.EnCnt = ENC_GetCounterValue(ENx);
		_motor->drv.Z_detect = SET;
	}

	/* Get Encoder counter value */
	_motor->drv.EnCnt = ENC_GetCounterValue(ENx);

	if (EncFlg.Bit.RotationDirection == 1)
	{
		/* CW */
		if (_motor->drv.EnCnt >= _motor->drv.EnCnt1)
		{
//			_motor->drv.EnCnt_dev = (q15_t)(_motor->drv.EnCnt - _motor->drv.EnCnt1);
			_motor->drv.EnCnt_dev = (int32_t)(_motor->drv.EnCnt - _motor->drv.EnCnt1);
		}
		else
		{
//			_motor->drv.EnCnt_dev = (q15_t)(_motor->para.enc.plsnum + _motor->drv.EnCnt - _motor->drv.EnCnt1);
			_motor->drv.EnCnt_dev = (int32_t)(_motor->para.enc.plsnum + _motor->drv.EnCnt - _motor->drv.EnCnt1);
		}
	}
	else
	{
		/* CCW */
		if (_motor->drv.EnCnt1 >= _motor->drv.EnCnt)
		{
//			_motor->drv.EnCnt_dev = (q15_t)((_motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
			_motor->drv.EnCnt_dev = (int32_t)((_motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
		}
		else
		{
//			_motor->drv.EnCnt_dev = (q15_t)((_motor->para.enc.plsnum + _motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
			_motor->drv.EnCnt_dev = (int32_t)((_motor->para.enc.plsnum + _motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
		}
	}

	_motor->drv.EnCnt1 = _motor->drv.EnCnt;

	/* Deside electrical angle. */
	if (EncFlg.Bit.ZPhaseDetectFlag == SET)
	{
		/* Calculation theta(electrical-angle) from encoder counter */
		_motor->drv.theta_enc = (uint32_t)(((uint64_t)_motor->drv.EnCnt * _motor->para.enc.pls2theta) + _motor->para.enc.deg_adjust);
	}
	else
	{
		if (_motor->HallCon->SensorFlg.hall_detect == SET)				/* Hall detected? */
		{
			if (_motor->HallCon->SensorFlg.ele_correct == SET)			/* Theta correction? */
			{
				/* z[ZT60xƂ̊pxɍXV */
				_motor->HallCon->SensorFlg.ele_correct = CLEAR;
				_motor->drv.theta_enc = ((uint32_t)_motor->HallCon->theta_hall << 16U);
			}
			else
			{
				/* z[ZTԂ̊pxGR[_pXŕ⊮ */
				_motor->drv.theta_enc += (int32_t)((int64_t)_motor->drv.EnCnt_dev * _motor->para.enc.pls2theta);
			}
		}
	}

	/* Cal omega(electrical-angle) from encoder counter deviation */
	_motor->drv.omega_enc_raw = (q15_t)(_motor->drv.EnCnt_dev * _motor->para.enc.pls2omega);
	_motor->drv.omega_enc_ave.word = calc_ave(_motor->drv.omega_enc_ave.word, _motor->drv.omega_enc_raw << 16 , cOMEGAENC_AVE);
	_motor->drv.omega_enc = _motor->drv.omega_enc_ave.half[1];

	if (_motor->drv.command.encoder == SET)
	{
		_motor->drv.omega.word = _motor->drv.omega_enc << 16;

		if (_motor->drv.vector_cmd.F_vcomm_theta == SET)
		{
			if (_motor->drv.vector_cmd.F_vcomm_omega == SET)
			{
				if ((_motor->HallCon->SensorFlg.hall_detect == SET)
				 || (_motor->HallCon->SensorFlg.enc_detect == SET))
				{
					/* z[ƃGR[_Zoς */
					_motor->drv.theta.word = _motor->drv.theta_enc;			/*  */
				}
				else
				{
					_motor->drv.theta.word = (_motor->drv.theta_com << 16);
				}
			}
			else
			{
				_motor->drv.theta.word += (_motor->drv.omega_com.half[1] * _motor->para.enc.ctrlprd) << 1;
			}
		}
		else
		{
			if (EncFlg.Bit.ZPhaseDetectFlag == 1)
			{
				/* Phase Z is detected */
				_motor->drv.theta.word = _motor->drv.theta_enc;
			}
			else
			{
				_motor->drv.theta.word = (_motor->drv.theta_com << 16);
			}
		}
	}
}
#endif

/**
 *********************************************************************************************
 * @brief		Detect _motor position
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
void D_Detect_Rotor_Position(vector_t* const _motor)
{

	q31_t temp32_r, temp32_Lq;
	q31_t temp32_OmegaErrI, temp32_OmegaErrP;
	q31_t omega_calc;

	/**
	 * @note	Voltage equation
	 * 			Vd = R*Id + Ld*pId - OMEGA-est*Lq + Ed
	 * 			  (p = d/dt, Id=Constant value -> pId = 0)
	 */
	if (_motor->drv.vector_cmd.F_vcomm_Edetect == SET)
	{
		/* Voltage drop of resistance	 V = R * I */
		temp32_r = (_motor->para.motor.r * (q31_t)_motor->drv.Id) << 1;


		/* Voltage drop of inductance	 V = 2PIfL * I */
                temp32_Lq = (((_motor->drv.omega.half[1] * _motor->para.motor.Lq))) >> __Q_FORMAT_MOTOR_L;		/* Q15 * __Q_FORMAT_MOTOR_L -> Q15 */
		temp32_Lq = (temp32_Lq * _motor->drv.Iq) << 1;								/* Q15 * Q15 -> Q31 *//*CNG2020*/

		/* d-axis induced voltage */
		/* Ed = Vd - R*Id + OMEGA-est*Lq */
		_motor->drv.Ed = (q15_t)((_motor->drv.Vd.word - temp32_r + temp32_Lq + cROUND_BIT15) >> 16);

		/**
		 * @note	OMEGAest = OMEGAref - K*Ed
		 */
		/* Set integration value */
#if defined(__POSITION_KI_RANGE)
		/* OmegaErr(I) * Hz_MAX = FIX_12 * Position_Ki * CtrlPrd * Ed * V_MAX */
		temp32_OmegaErrI = (_motor->para.pos.ki * (q31_t)_motor->drv.Ed) << 4;
#else
		/* OmegaErr(I) * Hz_MAX = FIX_15 * Position_Ki * CtrlPrd * Ed * V_MAX */
		temp32_OmegaErrI = (_motor->para.pos.ki * (q31_t)_motor->drv.Ed) << 1;
#endif

		if (_motor->drv.omega_com.half[1] >= 0)
		{
			_motor->drv.Ed_I = __QSUB(_motor->drv.Ed_I, temp32_OmegaErrI);
		}
		else
		{
			_motor->drv.Ed_I = __QADD(_motor->drv.Ed_I, temp32_OmegaErrI);
		}

		/* Set proportionality value */
#if defined(__POSITION_KP_RANGE)
		/* OmegaErr(P) * Hz_MAX = FIX_12 * Position_Kp * Ed * V_MAX */
		temp32_OmegaErrP = (_motor->para.pos.kp * (q31_t)_motor->drv.Ed) << 4;
#else
		/* OmegaErr(P) * Hz_MAX = FIX_15 * Position_Kp * Ed * V_MAX */
		temp32_OmegaErrP = (_motor->para.pos.kp * (q31_t)_motor->drv.Ed) << 1;
#endif

		if (_motor->drv.omega_com.half[1] >= 0)
		{
			_motor->drv.Ed_PI = __QSUB(_motor->drv.Ed_I, temp32_OmegaErrP);
		}
		else
		{
			_motor->drv.Ed_PI = __QADD(_motor->drv.Ed_I, temp32_OmegaErrP);
		}

		/* Speed calculation value = Speed command value + Error PI value */
		omega_calc = __QADD(_motor->drv.omega_com.word, _motor->drv.Ed_PI);

		if (_motor->drv.vector_cmd.F_vcomm_omega == SET)
		{
			if (_motor->drv.command.encoder == CLEAR)
			{
				_motor->drv.omega.word = omega_calc;
			}
		}
		else
		{
			/* Initialize of proportionality value, integration value and Omega */
			_motor->drv.omega.word = _motor->drv.omega_com.word;
			_motor->drv.Ed_I = 0;
			_motor->drv.Ed_PI = 0;
		}

		/* Position estimation */
		if (_motor->drv.vector_cmd.F_vcomm_theta == SET)
		{
			if (_motor->drv.command.encoder == CLEAR)
			{
				/* estimated THETAn = THETAn-1 + OMEGA * t */
				_motor->drv.theta.word += ((_motor->drv.omega.half[1] * _motor->para.pos.ctrlprd) << 1);
			}
		}
		else
		{
			_motor->drv.theta.word = (q31_t)(_motor->drv.theta_com << 16);
		}
	}
	else
	{
		_motor->drv.theta.word = (q31_t)(_motor->drv.theta_com << 16);
		_motor->drv.Ed = 0;
	}
}

/**
 *********************************************************************************************
 * @brief		Speed control ( Caricurate q-axis current )
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
void D_Control_Speed(vector_t* const _motor)
{
	/* Determination of R_Iq_ref */
	if (_motor->drv.vector_cmd.F_vcomm_current == SET)
	{
		q31_t temp32_PI;
		q31_t temp_iqref;

		/* Speed deviation */
		_motor->drv.omega_dev = (q15_t)(__QSUB(_motor->drv.omega_com.word, _motor->drv.omega.word) >> 16);

		/* Set proportionality value */
#if defined(__SPEED_KI_RANGE)
		temp32_PI = ((q31_t)_motor->para.spd.ki * _motor->drv.omega_dev) << 4;
#else
		temp32_PI = ((q31_t)_motor->para.spd.ki * _motor->drv.omega_dev) << 1;
#endif
		_motor->drv.Iq_ref_I.word = __QADD(_motor->drv.Iq_ref_I.word, temp32_PI);

		/* Limit value excess */
		if (_motor->drv.Iq_ref_I.word > _motor->para.iq_lim)
		{
			_motor->drv.Iq_ref_I.word = _motor->para.iq_lim;
		}
		else if (_motor->drv.Iq_ref_I.word < -_motor->para.iq_lim)
		{
			_motor->drv.Iq_ref_I.word = -_motor->para.iq_lim;
		}


		/* Set integration value */
#if defined(__SPEED_KP_RANGE)
		temp32_PI = (_motor->para.spd.kp * _motor->drv.omega_dev) << 4;
#else
		temp32_PI = (_motor->para.spd.kp * _motor->drv.omega_dev) << 1;
#endif
		temp_iqref = __QADD(_motor->drv.Iq_ref_I.word, temp32_PI);
		/* Limit value excess */
		if (temp_iqref > _motor->para.iq_lim)
		{
			_motor->drv.Iq_ref = (q15_t)(_motor->para.iq_lim >> 16);
		}
		else if (temp_iqref < -_motor->para.iq_lim)
		{
			_motor->drv.Iq_ref = (q15_t)(-_motor->para.iq_lim >> 16);
		}
		else
		{
			_motor->drv.Iq_ref = temp_iqref >> 16;
		}
	}
	else
	{
		/* Without Speed control. R_Iq_com -> R_Iq_ref */
		if (abs(_motor->drv.Iq_com.word) >= _motor->para.iq_lim)
		{
			/* Limit value excess */
			if (_motor->drv.Iq_com.word >= 0)
			{
				_motor->drv.Iq_ref = (q15_t)(_motor->para.iq_lim >> 16);
				_motor->drv.Iq_ref_I.word = _motor->para.iq_lim;
			}
			else
			{
				_motor->drv.Iq_ref = (q15_t)(-_motor->para.iq_lim >> 16);
				_motor->drv.Iq_ref_I.word = -_motor->para.iq_lim;
			}
		}
		else
		{
			/* Initialize of proportionality value and integration value */
			_motor->drv.Iq_ref = _motor->drv.Iq_com.half[1];
			_motor->drv.Iq_ref_I.word = _motor->drv.Iq_com.word;
		}

	}

	/* Determination of R_Id_ref */
	if (abs(_motor->drv.Id_com.word) >= _motor->para.id_lim)
	{
		/* Limit value excess */
		if (_motor->drv.Id_com.word >= 0)
		{
			_motor->drv.Id_ref = (q15_t)(_motor->para.id_lim >> 16);
		}
		else
		{
			_motor->drv.Id_ref = (q15_t)(-_motor->para.id_lim >> 16);
		}
	}
	else
	{
		_motor->drv.Id_ref = _motor->drv.Id_com.half[1];
	}

}

/**
 ********************************************************************************************
 * @brief		D_SetZeroCurrentByVE.
 *
 * @param[in]	vector_t* const	_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		none
 ********************************************************************************************
 */
void D_SetZeroCurrentByVE(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	if (_motor->stage.main == cStop)
	{
		if (_motor->shunt_type == c3shunt)
		{
			uint32_t	adc_ia, adc_ib, adc_ic;
			VE_GetCurrentAdcData(_ipdrv, &adc_ia, &adc_ib, &adc_ic);
			_motor->drv.Iao_ave.word += (((int32_t)(adc_ia << 16) - _motor->drv.Iao_ave.word) >> cIXO_AVE);
			_motor->drv.Ibo_ave.word += (((int32_t)(adc_ib << 16) - _motor->drv.Ibo_ave.word) >> cIXO_AVE);
			_motor->drv.Ico_ave.word += (((int32_t)(adc_ic << 16) - _motor->drv.Ico_ave.word) >> cIXO_AVE);
			VE_SetZeroCurrentData(_ipdrv, _motor->drv.Iao_ave.half[1], _motor->drv.Ibo_ave.half[1], _motor->drv.Ico_ave.half[1]);

		}
		else if (_motor->shunt_type == c1shunt)
		{
			ADC_Result	i_adc = {0};
			uint32_t	adc_ia;

			i_adc = ADC_GetConvertResult(_ipdrv->ADx, ADC_REG0);
			i_adc.All &= 0xfff0;

			_motor->drv.Iao_ave.word += (((int32_t)(i_adc.All << 16) - _motor->drv.Iao_ave.word) >> cIXO_AVE);

			adc_ia = _motor->drv.Iao_ave.half[1];
			VE_SetZeroCurrentData(_ipdrv, adc_ia, adc_ia, adc_ia);
		}
	}
}

/**
 *********************************************************************************************
 * @brief		D_SetZeroCurrent
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_SetZeroCurrent(vector_t* const _motor)
{
	if (_motor->stage.main == cStop)
	{
		if (_motor->shunt_type == c3shunt)
		{
			uint32_t	adc_ia, adc_ib, adc_ic;

			__disable_irq();
			adc_ia = _motor->drv.Ia_adc;
			adc_ib = _motor->drv.Ib_adc;
			adc_ic = _motor->drv.Ic_adc;
			__enable_irq();

			_motor->drv.Iao_ave.word += (((int32_t)(adc_ia << 16) - _motor->drv.Iao_ave.word) >> cIXO_AVE);
			_motor->drv.Ibo_ave.word += (((int32_t)(adc_ib << 16) - _motor->drv.Ibo_ave.word) >> cIXO_AVE);
			_motor->drv.Ico_ave.word += (((int32_t)(adc_ic << 16) - _motor->drv.Ico_ave.word) >> cIXO_AVE);
		}
		else if (_motor->shunt_type == c1shunt)
		{
			_motor->drv.Idco_ave.word += (((int32_t)(_motor->drv.Idc1_adc << 16) - _motor->drv.Idco_ave.word) >> cIXO_AVE);
		}
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		Check over current.
 *
 * @param		q15_t			_ovc	current of over error value.
 * @param		q15_t			_iu		current of u phase.
 * @param		q15_t			_iv		current of v phase.
 * @param		q15_t			_iw		current of w phase.
 *
 * @return		bool	TRUE:Over current / FALSE:Nomal
 *********************************************************************************************
 */
bool D_Check_OverCurrent(q15_t _ovc, q15_t _iu, q15_t _iv, q15_t _iw)
{
	if ((abs(_iu) > _ovc)
		|| (abs(_iv) > _ovc)
		|| (abs(_iw) > _ovc))
	{
		return (true);
	}
	else
	{
		return (false);
	}
}

/**
 *********************************************************************************************
 * @brief		D_Check_DetectCurrentError
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_Check_DetectCurrentError(vector_t* const _motor)
{
	uint32_t chkpls = _motor->para.chkpls;
	uint8_t error = OFF;

	if (_motor->shunt_type == c3shunt)
	{
		/* 3shunt */
		int32_t med = E_Med3(_motor->drv.DutyU, 
							 _motor->drv.DutyV, 
							 _motor->drv.DutyW);
		if (med >= chkpls)
		{
			error = ON;
		}
	}
	else
	{
		/* 1shunt */
		int32_t max, med, min;
		int32_t dif1, dif2;

		min = _motor->drv.DutyU;
		med = _motor->drv.DutyV;
		max = _motor->drv.DutyW;
		E_Sort3(&min, &med, &max);

		dif1 = (max - med) / 2;
		dif2 = (med - min) / 2;

		if (_motor->drv.command.modul == 0)
		{
			/* 3 phase modulation */
//			/* Check duty width */
//			if (min < chkpls)
//			{
//				error = ON;
//			}
			/* Check difference of duty width */
			if ((dif1 < chkpls) || (dif2 < chkpls))
			{
				error = ON;
			}
		}
		else
		{
			/* 2 phase modulation */
			/* Check duty width */
			if (med < chkpls)
			{
				error = ON;
			}
			/* Check difference of duty width */
			if (dif1 < chkpls)
			{
				error = ON;
			}
		}
	}

	if (error == ON)
	{
		_motor->drv.idetect_error = 1;
	}
	else
	{
		_motor->drv.idetect_error = 0;
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_GetMotorCurrentPowerVolt
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_GetMotorCurrentPowerVolt(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	/* Check conversion finishing flag. <PMDF = 0> */
//	while ((*((__I uint32_t*)BITBAND_PERI(&TSB_ADA->ST, B_PMDF))) == SET)
//	{
//	}
	AD_CheckFinishedConversion(_ipdrv);

	/* Get Power Supply Voltage */
//	_motor->drv.Vdc_adc = (q15_t)(*_motor->drv.ADxREG3 & 0xFFF0);
//	/* Power supply voltage is positive. 1bit right shift.(A sign is removed.) */
//	_motor->drv.Vdc = (_motor->drv.Vdc_adc >> 1);
	AD_GetVdcData(_motor);

	if ((_motor->para.TrgPosMd == cTRGPOS_1SHUNT_SECOND_HALF)  || (_motor->para.TrgPosMd == cTRGPOS_1SHUNT_FIRST_HALF))
	{

		if (_motor->para.TrgPosMd == cTRGPOS_1SHUNT_FIRST_HALF)
		{
			/* Acquire AD at the position in the first half of the PWM cycle. */
//			_motor->drv.Idc1_adc = (q15_t)(*_motor->drv.ADxREG1 & 0xFFF0);
//			_motor->drv.Idc2_adc = (q15_t)(*_motor->drv.ADxREG0 & 0xFFF0);
			AD_GetIdcFirstHalf(_motor);
		}
		else /* if (_motor->para.TrgPosMd == cTRGPOS_1SHUNT_SECOND_HALF) */
		{
			/* Acquire AD at the position in the second half of the PWM cycle. */
//			_motor->drv.Idc1_adc = (q15_t)(*_motor->drv.ADxREG0 & 0xFFF0);
//			_motor->drv.Idc2_adc = (q15_t)(*_motor->drv.ADxREG1 & 0xFFF0);
			AD_GetIdcSecondHalf(_motor);
		}

		switch(_motor->drv.Sector1)
		{
			case 0:
				_motor->drv.Ic_raw = (q15_t)(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc1_adc);
				_motor->drv.Ia_raw = (q15_t)(-(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc2_adc));
				_motor->drv.Ib_raw = (q15_t)(-_motor->drv.Ia_raw - _motor->drv.Ic_raw);
				break;
			case 1:
				_motor->drv.Ic_raw = (q15_t)(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc1_adc);
				_motor->drv.Ib_raw = (q15_t)(-(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc2_adc));
				_motor->drv.Ia_raw = (q15_t)(-_motor->drv.Ib_raw -_motor->drv.Ic_raw);
				break;
			case 2:
				_motor->drv.Ia_raw = (q15_t)(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc1_adc);
				_motor->drv.Ib_raw = (q15_t)(-(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc2_adc));
				_motor->drv.Ic_raw = (q15_t)(-_motor->drv.Ia_raw - _motor->drv.Ib_raw);
				break;
			case 3:
				_motor->drv.Ia_raw = (q15_t)(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc1_adc);
				_motor->drv.Ic_raw = (q15_t)(-(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc2_adc));
				_motor->drv.Ib_raw = (q15_t)(-_motor->drv.Ia_raw - _motor->drv.Ic_raw);
				break;
			case 4:
				_motor->drv.Ib_raw = (q15_t)(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc1_adc);
				_motor->drv.Ic_raw = (q15_t)(-(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc2_adc));
				_motor->drv.Ia_raw = (q15_t)(-_motor->drv.Ib_raw - _motor->drv.Ic_raw);
				break;
			case 5:
				_motor->drv.Ib_raw = (q15_t)(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc1_adc);
				_motor->drv.Ia_raw = (q15_t)(-(_motor->drv.Idco_ave.half[1] - _motor->drv.Idc2_adc));
				_motor->drv.Ic_raw = (q15_t)(-_motor->drv.Ia_raw - _motor->drv.Ib_raw);
				break;
			default:
				break;
		}
	}
	else /* if (_motor->para.TrgPosMd == cTRGPOS_3SHUNT) */
	{
		/* Get phase current */
		switch(_motor->drv.Sector1)
		{
			case 0:
			case 5:
				/* Sector 0,5 */
				_motor->drv.Ia_adc = (q15_t)(*_motor->drv.ADxREG2 & 0xFFF0);
				_motor->drv.Ib_adc = (q15_t)(*_motor->drv.ADxREG0 & 0xFFF0);
				_motor->drv.Ic_adc = (q15_t)(*_motor->drv.ADxREG1 & 0xFFF0);
				_motor->drv.Ib_raw = (q15_t)(_motor->drv.Ibo_ave.half[1] - _motor->drv.Ib_adc);
				_motor->drv.Ic_raw = (q15_t)(_motor->drv.Ico_ave.half[1] - _motor->drv.Ic_adc);
				_motor->drv.Ia_raw = (q15_t)(-_motor->drv.Ib_raw - _motor->drv.Ic_raw);
				break;
			case 1:
			case 2:
				/* Sector1,2 */
				_motor->drv.Ia_adc = (q15_t)(*_motor->drv.ADxREG1 & 0xFFF0);
				_motor->drv.Ib_adc = (q15_t)(*_motor->drv.ADxREG2 & 0xFFF0);
				_motor->drv.Ic_adc = (q15_t)(*_motor->drv.ADxREG0 & 0xFFF0);
				_motor->drv.Ia_raw = (q15_t)(_motor->drv.Iao_ave.half[1] - _motor->drv.Ia_adc);
				_motor->drv.Ic_raw = (q15_t)(_motor->drv.Ico_ave.half[1] - _motor->drv.Ic_adc);
				_motor->drv.Ib_raw = (q15_t)(-_motor->drv.Ic_raw - _motor->drv.Ia_raw);
				break;
			case 3:
			case 4:
				/* Sector3,4 */
				_motor->drv.Ia_adc = (q15_t)(*_motor->drv.ADxREG0 & 0xFFF0);
				_motor->drv.Ib_adc = (q15_t)(*_motor->drv.ADxREG1 & 0xFFF0);
				_motor->drv.Ic_adc = (q15_t)(*_motor->drv.ADxREG2 & 0xFFF0);
				_motor->drv.Ia_raw = (q15_t)(_motor->drv.Iao_ave.half[1] - _motor->drv.Ia_adc);
				_motor->drv.Ib_raw = (q15_t)(_motor->drv.Ibo_ave.half[1] - _motor->drv.Ib_adc);
				_motor->drv.Ic_raw = (q15_t)(-_motor->drv.Ia_raw - _motor->drv.Ib_raw);
				break;
			default:
				break;
		}
	}

	if ((_motor->stage.itr == ciStop) || (_motor->drv.idetect_error == 0))
	{
		_motor->drv.Ia = _motor->drv.Ia_raw;
		_motor->drv.Ib = _motor->drv.Ib_raw;
		_motor->drv.Ic = _motor->drv.Ic_raw;
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_InputTransformation
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_InputTransformation(vector_t* const _motor)
{
	if (_motor->drv.vector_cmd.F_vcomm_Idetect == SET)
	{
		/* When the current detection can not be performed normally, conversion is not performed and Id and Iq are not updated. */
		if (_motor->drv.idetect_error == 0)
		{
			/* abc/alpha_beta conversion (Clarke.t) */
			E_Clarke(_motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic, &_motor->drv.Ialpha, &_motor->drv.Ibeta);

			/* alpha_beta/dq conversion (Park.t) */
			E_Park(_motor->drv.Ialpha, _motor->drv.Ibeta, _motor->drv.theta.half[1], &_motor->drv.Id, &_motor->drv.Iq);
		}
	}
	else
	{
		_motor->drv.Id = 0;
	  	_motor->drv.Iq = 0;
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_Control_Current
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_Control_Current(vector_t* const _motor)
{
	if (_motor->drv.vector_cmd.F_vcomm_volt == SET)
	{
		if ((_motor->stage.itr == ciInitposition_v) || (_motor->stage.itr == ciForce_v))
		{
			_motor->drv.Vd.word = _motor->drv.Vd_I.word = _motor->drv.Vd_out;
			_motor->drv.Vq.word = _motor->drv.Vq_I.word = 0;
		}
		else
		{
			q31_t	temp_PI;

			/* Id deviation */
			_motor->drv.Id_dev = (q15_t)(_motor->drv.Id_ref - _motor->drv.Id);
#if defined(__CURRENT_DKI_RANGE)
			temp_PI = ((q31_t)_motor->para.crt.dki * _motor->drv.Id_dev) << 4;
#else
			temp_PI = ((q31_t)_motor->para.crt.dki * _motor->drv.Id_dev) << 1;
#endif
			_motor->drv.Vd_I.word = __QADD(_motor->drv.Vd_I.word, temp_PI);

#if defined(__CURRENT_DKP_RANGE)
			temp_PI = ((q31_t)_motor->para.crt.dkp * _motor->drv.Id_dev) << 4;
#else
			temp_PI = ((q31_t)_motor->para.crt.dkp * _motor->drv.Id_dev) << 1;
#endif
			_motor->drv.Vd.word = __QADD(_motor->drv.Vd_I.word, temp_PI);

			/* Iq deviation */
			_motor->drv.Iq_dev = (q15_t)(_motor->drv.Iq_ref - _motor->drv.Iq);
#if defined(__CURRENT_QKI_RANGE)
			temp_PI = ((q31_t)_motor->para.crt.qki * _motor->drv.Iq_dev) << 4;
#else
			temp_PI = ((q31_t)_motor->para.crt.qki * _motor->drv.Iq_dev) << 1;
#endif
			_motor->drv.Vq_I.word = __QADD(_motor->drv.Vq_I.word, temp_PI);

#if defined(__CURRENT_QKP_RANGE)
			temp_PI = ((q31_t)_motor->para.crt.qkp * _motor->drv.Iq_dev) << 4;
#else
			temp_PI = ((q31_t)_motor->para.crt.qkp * _motor->drv.Iq_dev) << 1;
#endif
			_motor->drv.Vq.word = __QADD(_motor->drv.Vq_I.word, temp_PI);
		}

#if defined(__USE_DECOUPLING)
		D_Decouple(&_motor->Decouple, _motor->drv.omega.half[1], _motor->drv.Iq_ref, _motor->drv.Id_ref, _motor->para.motor.Lq, _motor->para.motor.Ld);
		_motor->drv.Vd.word += (_motor->Decouple.Vde << 16);
		_motor->drv.Vq.word += (_motor->Decouple.Vqe << 16);
#endif /* defined(__USE_DECOUPLING) */
	}
	else
	{
		/* Without current control. Vd_com,Vq_com -> Vd,Vq */
		_motor->drv.Vd.word = _motor->drv.Vd_I.word = _motor->drv.Vd_com.word;
		_motor->drv.Vq.word = _motor->drv.Vq_I.word = _motor->drv.Vq_com.word;
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_OutputTransformation
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_OutputTransformation(vector_t* const _motor)
{
	if (_motor->drv.vector_cmd.F_vcomm_onoff == CLEAR)
	{
		_motor->drv.Sector = _motor->drv.Sector1 = 0;
		_motor->drv.DutyU  = _motor->drv.DutyV   = _motor->drv.DutyW = 0;
	}
	else
	{
		/*
			It converts from dq-axis voltage (Vd, Vq) to alpha_beta-axis voltage (Valpha, Vbeta) by reverse park conversion.
		*/
		E_InvPark(_motor->drv.Vd.word, _motor->drv.Vq.word, _motor->drv.theta.half[1], &_motor->drv.Valpha.word, &_motor->drv.Vbeta.word);

		/*
			Calculate sectors from alpha_beta axis voltage (Valpha, Vbeta). 
			Sectors before calculation are saved in the last sector value and used for motor current acquisition.
		*/
		_motor->drv.Sector1 = _motor->drv.Sector;
		_motor->drv.Sector = D_CalSector(_motor->drv.Valpha.word, _motor->drv.Vbeta.word);

		/*
			Convert from alpha_beta axis voltage (Valpha, Vbeta) to 3-phase duty (DutyV, DutyW, DutyW) by space vector modulation or reverse clark conversion.
		*/
		if (_motor->para.PhCvMd == cPHCVMD_SVM)
		{
			D_SVM(_motor->drv.Valpha.word, _motor->drv.Vbeta.word, _motor->drv.Vdc, _motor->drv.Sector, _motor->drv.command.modul,
				  (uint16_t *)&_motor->drv.DutyU, (uint16_t *)&_motor->drv.DutyV, (uint16_t *)&_motor->drv.DutyW);

#if defined(__USE_DEADTIME)
			D_Deadtime_Control(&_motor->Deadtime, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic, _motor->drv.idetect_error);
			_motor->drv.DutyU += _motor->Deadtime.u_duty;
			_motor->drv.DutyV += _motor->Deadtime.v_duty;
			_motor->drv.DutyW += _motor->Deadtime.w_duty;
#endif /* defined(__USE_DEADTIME) */
		}
		else if (_motor->para.PhCvMd == cPHCVMD_I_CLARKE)
		{
			/* InvClarke() not supported */
		}
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 ********************************************************************************************
 * @brief		To determine the output mode.
 *
 * @param[in]	vector_t* const	_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		none
 ********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_DecisionOutputMode(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	uint32_t tmp_mdout;

	if (_motor->drv.vector_cmd.F_vcomm_onoff == CLEAR)
	{
		tmp_mdout = cMDOUT_OFF;
	}
	else
	{
		if (_motor->drv.state.all == 0)						/* No error? */
		{
			if (_motor->stage.itr == ciBootstrap)
			{
				tmp_mdout = cMDOUT_BOOTSTRAP;
			}
			else
			{
				tmp_mdout = cMDOUT_ON;
			}
		}
		else
		{
			tmp_mdout = cMDOUT_OFF;
		}
	}

	PMD_SetOutputMode(_ipdrv, tmp_mdout);

}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_CalSector
 *
 * @param		q31_t 		_valpha		alpha-axis Valpha
 * @param		q31_t 		_vbeta		beta-axis Vbeta
 *
 * @return		sector	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
uint8_t D_CalSector(q31_t _valpha, q31_t _vbeta)
{
	#define	cCAL_1_DIV_SQRT3	(int32_t)(FIX_15 * 1 / 1.73205f)	/* Q15 1/sqrt3 */

  	uint8_t sector = 0;
	q31_t temp_vbeta = ((((int64_t)abs(_vbeta)) * cCAL_1_DIV_SQRT3) >> 15);

	if ((_valpha >= 0) && (_vbeta >= 0))
	{
		sector = (_valpha >= temp_vbeta)? 0: 1;
	}
	else if ((_valpha < 0) && (_vbeta >= 0))
	{
		sector = (abs(_valpha) < temp_vbeta)? 1: 2;
	}
	else if ((_valpha < 0) && (_vbeta < 0))
	{
		sector = (abs(_valpha) < temp_vbeta)? 4: 3;
	}
	else if ((_valpha >= 0) && (_vbeta < 0))
	{
		sector = (_valpha < temp_vbeta)? 4: 5;
	}
	return(sector);
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		Limitation.
 *
 * @param		int64_t			_indata			Q31 input data(Double precision).
 * @param		int32_t			_limlo			Q31 lower limit value.
 * @param		int32_t			_limup			Q31 upper limit value.
 *
 * @return		int32_t			Q31
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
static int32_t Limit_sub(int64_t _indata, int32_t _limlo, int32_t _limup)
{
  	int32_t ret = (int32_t)_indata;
 
	if (_indata > (int64_t)_limup)
	{
		ret = _limup;
	}
	if (_indata < (int64_t)_limlo)
	{
		ret = _limlo;
	}
	return (ret);
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_SVM
 *
 * @param		q31_t 		_valpha		alpha-axis Valpha
 * @param		q31_t 		_vbeta		beta-axis Vbeta
 * @param		q15_t	 	_vdc		DC voltage
 * @param		uint8_t 	_sector		sector
 * @param		uint8_t 	_modul		modul
 * @param		uint16_t* 	_p_vu		U phase duty ratio storage address
 * @param		uint16_t* 	_p_vv		V phase duty ratio storage address
 * @param		uint16_t* 	_p_vw		W phase duty ratio storage address
 *
 * @return		none	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_SVM(q31_t _valpha, q31_t _vbeta, q15_t _vdc, uint8_t _sector, uint8_t _modul, uint16_t* _p_vu, uint16_t* _p_vv, uint16_t* _p_vw)
{
	#define cSVM_K				(int64_t)((1L<<15) * 3.0f / 2.0f)			/* Q15 3/2 */
	#define	cSVM_1_DIV_SQRT3	(int64_t)((1L<<15) * 1.0f / 1.7320508f)		/* Q15 1/sqrt3 */
	#define	cSVM_2_DIV_SQRT3	(int64_t)((1L<<15) * 2.0f / 1.7320508f)		/* Q15 2/sqrt3 */

	#define cSVM_100_PERCENT	(int32_t)(((1LL<<31) * 1) - 1) /* Q31 1.0 */

	int32_t temp_1, temp_2;
	q31_t d0, d1, d2;

	switch (_sector)
	{
		case 0:
			/* Sector0 */
			/* D1 = k * (Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * ( 2/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)(__QSUB(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15)));
			temp_2 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D1 + D2 + (D0 / 2) */
				/* V = D2 + (D0 / 2) */
				/* W = D0 / 2 */
				*_p_vu = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vv = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vw = (uint16_t)((d0 / 2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D1 + D2 */
				/* V = D2 */
				/* W = 0 */
				*_p_vu = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vv = (uint16_t)(d2 >> 16);
				*_p_vw = 0;
			}
			break;

		case 1:
			/* Sector1 */
			/* D1 = k * (-Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)__QADD((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			temp_2 = (int32_t)__QADD(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D2 + (D0 / 2) */
				/* V = D1 + D2 + (D0 / 2) */
				/* W = D0 / 2 */
				*_p_vu = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vv = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vw = (uint16_t)((d0 / 2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D2 */
				/* V = D1 + D2 */
				/* W = 0 */
				*_p_vu = (uint16_t)(d2 >> 16);
				*_p_vv = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vw = 0;
			}
			break;

		case 2:
			/* Sector2 */
			/* D1 = k * ( 2/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (-Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			temp_2 = (int32_t)__QSUB((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D0 / 2 */
				/* V = D1 + D2 + (D0 / 2) */
				/* W = D2 + (D0 / 2) */
				*_p_vu = (uint16_t)((d0 / 2) >> 16);
				*_p_vv = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vw = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = 0 */
				/* V = D1 + D2 */
				/* W = D2 */
				*_p_vu = 0;
				*_p_vv = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vw = (uint16_t)(d2 >> 16);
			}
			break;
		
		case 3:
			/* Sector3 */
			/* D1 = k * (-2/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (-Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			temp_2 = (int32_t)__QADD((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (-temp_1 /_vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D0 / 2*/
				/* V = D2 + (D0 / 2) */
				/* W = D1 + D2 + (D0 / 2) */
				*_p_vu = (uint16_t)((d0 / 2) >> 16);
				*_p_vv = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vw = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = 0 */
				/* V = D2 */
				/* W = D1 + D2 */
				*_p_vu = 0;
				*_p_vv = (uint16_t)(d2 >> 16);
				*_p_vw = (uint16_t)(__QADD(d1, d2) >> 16);
			}
			break;

		case 4:
			/* Sector4 */
			/* D1 = k * (-Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)__QSUB((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			temp_2 = (int32_t)__QSUB(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D2 + (D0 / 2) */
				/* V = D0 / 2 */
				/* W = D1 + D2 + (D0 / 2) */
				*_p_vu = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vv = (uint16_t)((d0 / 2) >> 16);
				*_p_vw = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D2 */
				/* V = 0 */
				/* W = D1 + D2 */
				*_p_vu = (uint16_t)(d2 >> 16);
				*_p_vv = 0;
				*_p_vw = (uint16_t)(__QADD(d1, d2) >> 16);
			}
			break;

		case 5:
			/* Sector5 */
			/* D1 = k * (Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * ( -2/sqrt3 Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)__QADD(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			temp_2 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (-temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)Limit_sub(__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2), 0, cSVM_100_PERCENT);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D1 + D2 + (D0 / 2) */
				/* V = D0 / 2 */
				/* W = D2 + (D0 / 2) */
				*_p_vu = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vv = (uint16_t)((d0 / 2) >> 16);
				*_p_vw = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D1 + D2 */
				/* V = 0 */
				/* W = D2 */
				*_p_vu = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vv = 0;
				*_p_vw = (uint16_t)(d2 >> 16);
			}
			break;
		default:
			break;
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_CalTrgTiming
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none	
 *********************************************************************************************
 */
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
void D_CalTrgTiming(vector_t* const _motor)
{
	if (_motor->drv.vector_cmd.F_vcomm_onoff == CLEAR)
	{
		_motor->drv.AdTrg0 = _motor->drv.AdTrg1  = 0;
	}
	else
	{
		uint32_t  dmin, dmid, dmax;

		switch(_motor->para.TrgPosMd)
		{
			case cTRGPOS_1SHUNT_SECOND_HALF:
				dmid = E_Med3(_motor->drv.DutyU, 
							  _motor->drv.DutyV, 
							  _motor->drv.DutyW);
				dmax = E_Max3(_motor->drv.DutyU, 
							  _motor->drv.DutyV, 
							  _motor->drv.DutyW);

				/* 2phase and 3phase modulation */
				 _motor->drv.AdTrg0 = dmid + _motor->para.TrgComp;
				 _motor->drv.AdTrg1 = dmax + _motor->para.TrgComp;
				break;

			case cTRGPOS_1SHUNT_FIRST_HALF:
				dmid = E_Med3(_motor->drv.DutyU, 
							  _motor->drv.DutyV, 
							  _motor->drv.DutyW);
				dmin = E_Min3(_motor->drv.DutyU, 
							  _motor->drv.DutyV, 
							  _motor->drv.DutyW);

				if (_motor->drv.command.modul == 0)
				{
					/* 3phase modulation */
					_motor->drv.AdTrg0 = (-dmid) + _motor->para.TrgComp;
					_motor->drv.AdTrg1 = (-dmin) + _motor->para.TrgComp;
				}
				else
				{
					/* 2phase modulation */
					_motor->drv.AdTrg0 = (-dmid) + _motor->para.TrgComp;
					_motor->drv.AdTrg1 = dmid + _motor->para.TrgComp;
				}
				break;

			case cTRGPOS_3SHUNT:
			{
				int32_t	trgtmp;
				
				if (_motor->para.TrgComp > 0)
				{
					trgtmp = (1L<<15) + _motor->para.TrgComp;
					if (trgtmp > (1L<<15))	trgtmp -= (1L<<16);
				}
				else
				{
					trgtmp = (1L<<15);
				}

				_motor->drv.AdTrg0 = trgtmp;
				_motor->drv.AdTrg1 = trgtmp;
			}
				break;

			default:
				break;
		}
	}
}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */

/**
 *********************************************************************************************
 * @brief		D_Check_DetectCurrentError_1shunt
 *
 * @param		uint32_t	_pwma			U phase Duty
 * @param		uint32_t	_pwmb			V phase Duty
 * @param		uint32_t	_pwmc			W phase Duty
 * @param		uint16_t	_chkplswidth	Current false detection pulse width
 * @param		int			_sfhpwm			Shift PWM mode: 0:OFF, 1:ON
 * @param		int			_modu			PWM modulation mode: c3PHASE or c2PHASE
 * @param		int			_sector			Sector[0-11]
 * @param		uint32_t	_mdprd			PWM career
 *
 * @return		int			0:Correctly detection 1:False detection
 *********************************************************************************************
 */
#if defined(__A_VE) || defined(__VE_P) || defined(__VE)
int D_Check_DetectCurrentError_1shunt(uint32_t _pwma, uint32_t _pwmb, uint32_t _pwmc, uint16_t _chkplswidth, int _sfhpwm, int _modu, int _sector, uint32_t _mdprd)
{
	int32_t	dif1, dif2, dif3;
	int32_t	difmin, difmid, difmax;
	int32_t	chkpls;

	if (_sfhpwm == 1)
	{
		/* Shift PWM */
		switch(_sector)
		{
			case 11:
			case 0:
			case 1:
			case 2:
				/* V Phase Centor OFF */
				_pwmb = _mdprd - _pwmb;
				break;

			case 3:
			case 4:
			case 5:
			case 6:
				/* W Phase Centor OFF */
				_pwmc = _mdprd - _pwmc;
				break;

			case 7:
			case 8:
			case 9:
			case 10:
				/* U Phase Centor OFF */
				_pwma = _mdprd - _pwma;
				break;
			default:
				break;
		}
	}

	/* Calculate the difference of Duty. */
	dif1 = abs(_pwmc - _pwma);
	dif2 = abs(_pwmb - _pwmc);
	dif3 = abs(_pwma - _pwmb);

	/* Decision min value */
	difmin = dif1;
	if (dif2 < difmin)
	{
		difmin = dif2;
	}
	if (dif3 < difmin)
	{
		difmin = dif3;
	}

	/* Decision max value */
	difmax = (dif1 + dif2 + dif3) / 2;

	/* Decision mid value */
	difmid = difmax - difmin;

	/* Calculation detection width  */
	if (_sfhpwm == 1)
	{
		/* Shift PWM */
		chkpls = difmid;
	}
	else if(_modu == c2PHASE)
	{
		/* Normal PWM 2Phase */
		if (((_sector & 0x3) == 0) || ((_sector & 0x3) == 3))
		{
			chkpls = difmin * 2;
			if(difmid < chkpls)
			{
				chkpls = difmid;
			}
		}
		else
		{
			chkpls = difmin;
		}
	}
	else
	{
		/* Normal PWM 3Phase */
		chkpls = difmin;
	}

	/* Check pulse width */
	if (chkpls < _chkplswidth)
	{
//                TSB_PB_DATA_PB7 = 0;
		return(1);
	}
	else
	{
//                TSB_PB_DATA_PB7 = 1;
		return(0);
	}
}
#endif /* defined(__A_VE) || defined(__VE_P) || defined(__VE) */

/**
 *********************************************************************************************
 * @brief		D_Check_DetectCurrentError_3shunt
 *
 * @param		uint32_t	_duty			Medium duty value
 * @param		uint16_t	_chkplswidth	Current false detection pulse width
 *
 * @return		int	
 ***********************rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr**********************************************************************
 */
#if defined(__A_VE) || defined(__VE_P) || defined(__VE)
int D_Check_DetectCurrentError_3shunt(uint32_t _duty, uint16_t _chkplswidth)
{
	if (_duty >= _chkplswidth)
	{
		return(1);
	}
	else
	{
		return(0);
	}
}
#endif /* defined(__A_VE) || defined(__VE_P) || defined(__VE) */

/*********************************** END OF FILE ******************************/
