/**
 ****************************************************************************
 * @file:	interrupt.c
 * @brief:	Interrupt Functions for All Soft
 * @version	V1.2
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>

#include "ipdefine.h"
#include "mcuip_drv.h"
#include "D_Driver.h"
#include "usercon.h"
#include "interrupt.h"

//#define MOTOR_OUTPUT	0
#define MOTOR_OUTPUT	1

/* for debug */
#if	defined(__MOTOR_DBGOUT_IDET)

 #if defined(__USE_VE_CH2) || defined(__USE_VE_CH1) || defined(__USE_VE_CH0)


	#define	DEBUG_IDET_ERROR(x)		debug_idet_error_ve(x)
	void	debug_idet_error_ve(const ipdrv_t* const _ipdrv)
	{
		if (VE_GetPLSLF(_ipdrv) == ve_plslf_state_on)
		{
			TP_IDET_HIGH;
		}
		else
		{
			TP_IDET_LOW;
		}
	}/*CNG2020*/

#else
	#define	DEBUG_IDET_ERROR(x)			debug_idet_error(x)
	void	debug_idet_error(vector_t* const _motor)
	{
		if (_motor->drv.idetect_error != 0)
		{
			TP_IDET_HIGH;
		}
		else
		{
			TP_IDET_LOW;
		}
	}
 #endif /* defined(__USE_VE_CH2) || defined(__USE_VE_CH1) || defined(__USE_VE_CH0) */

#else
	#define	DEBUG_IDET_ERROR(x)
#endif	/* defined(__MOTOR_DBGOUT_IDET) */


/**
 *********************************************************************************************
 * @brief		INT_interval_4kHz
 *
 * @return		none
 *********************************************************************************************
 */
void INT_interval_4kHz(void)
{
	M_Main_Counter++;									/* Count for main routine Period */

}

/**
 ********************************************************************************************
 * @brief		Vector Control Interruption by VE for ch0.
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH0) && defined(__USE_VE_CH0)
void INT_VectorControlByVE_CH0(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	DEBUG_IDET_ERROR(_ipdrv);

#if (__FIXED_VDC_CH0 == 1)								/* 1: Vdc from constant data */
	/***** Set Vdc data to VE *****/
	VE_SetVDCreg(_ipdrv, (q15_t)(FIX_15 * cVDC_CH0 / cV_MAX_CH0));
#endif
        
	/***** Get data from VE register *****/
        
        
	VE_GetdataFromVEreg(_ipdrv, _motor);

	/***** Check over current *****/
	VE_GetPhaseCurrent(_ipdrv, &_motor->drv.Ia, &_motor->drv.Ib, &_motor->drv.Ic);
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}

	/***** Check error voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH0 / cV_MAX_CH0)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH0 / cV_MAX_CH0) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH0)
		/***** Encoder *****/
		H_Encoder(_motor, TSB_EN0);
#endif /* __USE_ENCODER_CH0 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH0)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}
	}

	/**** Set data to VE register *****/
	/* Set Shift pwm change speed. for 1shunt */
	VE_SetSpwmMode(_ipdrv, _motor->drv.command.spwm);

	/* Set modulation type */
	VE_SetModulType(_ipdrv, _motor->drv.command.modul);

	switch (_motor->stage.itr)
	{
			/*--- Stop Stage -----------------------------*/
		case ciStop:
			VE_SetdataToVEreg_Stop(_ipdrv, _motor);
			break;

			/*--- Bootstrap Stage ------------------------*/
		case ciBootstrap:
			VE_SetdataToVEreg_Bootstrap(_ipdrv, _motor);
			break;

			/*--- Position Stage (Current type) ----------*/
		case ciInitposition_i:
			VE_SetdataToVEreg_Initposition_i(_ipdrv, _motor);
			break;

			/*--- Position Stage (Voltage type) ----------*/
		case ciInitposition_v:
			VE_SetdataToVEreg_Initposition_v(_ipdrv, _motor);
			break;

			/*--- Force Stage (Current type) -------------*/
		case ciForce_i:
			VE_SetdataToVEreg_Force_i(_ipdrv, _motor);
			break;

			/*--- Force Stage (Voltage type) -------------*/
		case ciForce_v:
			VE_SetdataToVEreg_Force_v(_ipdrv, _motor);
			break;

			/*--- ChangeUp Stage -------------------------*/
		case ciChange_up:
			VE_SetdataToVEreg_Change_up(_ipdrv, _motor);
			break;

			/*--- Steady_A Stage ---------------------------*/
		case ciSteady_A:
			VE_SetdataToVEreg_Steady_A(_ipdrv, _motor);
			break;

			/*--- Emergency Stage ------------------------*/
		case ciEmergency:
			VE_SetdataToVEreg_Emergency(_ipdrv, _motor);
			break;
			/*--- Fail-safe ------------------------*/
		default:
			VE_SetdataToVEreg_Stop(_ipdrv, _motor);
			break;
	}

#if defined(__USE_DAC)
	UiOutDataStartByVE(0);							/* Dac output for debug */
#endif /* __USE_DAC */

	VE_Start(_ipdrv);

}
#endif /* defined(__CONTROL_MOTOR_CH0) && defined(__USE_VE_CH0) */

/**
 ********************************************************************************************
 * @brief		Vector Control Interruption by VE for ch1.
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH1) && defined(__USE_VE_CH1)
void INT_VectorControlByVE_CH1(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	DEBUG_IDET_ERROR(_ipdrv);

#if (__FIXED_VDC_CH1 == 1)								/* 1: Vdc from constant data */
	/***** Set Vdc data to VE *****/
	VE_SetVDCreg(_ipdrv, (q15_t)(FIX_15 * cVDC_CH1 / cV_MAX_CH1));
#endif

	/***** Get data from VE register *****/
	VE_GetdataFromVEreg(_ipdrv, _motor);

	/***** Check over current *****/
	VE_GetPhaseCurrent(_ipdrv, &_motor->drv.Ia, &_motor->drv.Ib, &_motor->drv.Ic);
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}

	/***** Check error voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH1 / cV_MAX_CH1)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH1 / cV_MAX_CH1) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH1)
		/***** Encoder *****/
		H_Encoder(_motor, TSB_EN1);
#endif /* __USE_ENCODER_CH1 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH1)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}
	}

	/**** Set data to VE register *****/
	/* Set Shift pwm change speed. for 1shunt */
	VE_SetSpwmMode(_ipdrv, _motor->drv.command.spwm);

	/* Set modulation type */
	VE_SetModulType(_ipdrv, _motor->drv.command.modul);

	switch (_motor->stage.itr)
	{
			/*--- Stop Stage -----------------------------*/
		case ciStop:
			VE_SetdataToVEreg_Stop(_ipdrv, _motor);
			break;

			/*--- Bootstrap Stage ------------------------*/
		case ciBootstrap:
			VE_SetdataToVEreg_Bootstrap(_ipdrv, _motor);
			break;

			/*--- Position Stage (Current type) ----------*/
		case ciInitposition_i:
			VE_SetdataToVEreg_Initposition_i(_ipdrv, _motor);
			break;

			/*--- Position Stage (Voltage type) ----------*/
		case ciInitposition_v:
			VE_SetdataToVEreg_Initposition_v(_ipdrv, _motor);
			break;

			/*--- Force Stage (Current type) -------------*/
		case ciForce_i:
			VE_SetdataToVEreg_Force_i(_ipdrv, _motor);
			break;

			/*--- Force Stage (Voltage type) -------------*/
		case ciForce_v:
			VE_SetdataToVEreg_Force_v(_ipdrv, _motor);
			break;

			/*--- ChangeUp Stage -------------------------*/
		case ciChange_up:
			VE_SetdataToVEreg_Change_up(_ipdrv, _motor);
			break;

			/*--- Steady_A Stage ---------------------------*/
		case ciSteady_A:
			VE_SetdataToVEreg_Steady_A(_ipdrv, _motor);
			break;

			/*--- Emergency Stage ------------------------*/
		case ciEmergency:
			VE_SetdataToVEreg_Emergency(_ipdrv, _motor);
			break;
			/*--- Fail-safe ------------------------*/
		default:
			VE_SetdataToVEreg_Stop(_ipdrv, _motor);
			break;
	}

#if defined(__USE_DAC)
	UiOutDataStartByVE(1);									/* Dac output for debug */
#endif /* __USE_DAC */

	VE_Start(_ipdrv);

}
#endif /* defined(__CONTROL_MOTOR_CH1) && defined(__USE_VE_CH1) */

/**
 ********************************************************************************************
 * @brief		Vector Control Interruption by VE for ch2.
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH2) && defined(__USE_VE_CH2)
void INT_VectorControlByVE_CH2(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	DEBUG_IDET_ERROR(_ipdrv);

#if (__FIXED_VDC_CH2 == 1)								/* 1: Vdc from constant data */
	/***** Set Vdc data to VE *****/
	VE_SetVDCreg(_ipdrv, (q15_t)(FIX_15 * cVDC_CH2 / cV_MAX_CH2));
#endif

	/***** Get data from VE register *****/
	VE_GetdataFromVEreg(_ipdrv, _motor);

	/***** Check over current *****/
	VE_GetPhaseCurrent(_ipdrv, &_motor->drv.Ia, &_motor->drv.Ib, &_motor->drv.Ic);
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}

	/***** Check error voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH2 / cV_MAX_CH2)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH2 / cV_MAX_CH2) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH2)
		/***** Encoder *****/
		H_Encoder(_motor, TSB_EN2);
#endif /* __USE_ENCODER_CH2 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH2)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}
	}

	/**** Set data to VE register *****/
	/* Set Shift pwm change speed. for 1shunt */
	VE_SetSpwmMode(_ipdrv, _motor->drv.command.spwm);

	/* Set modulation type */
	VE_SetModulType(_ipdrv, _motor->drv.command.modul);

	switch (_motor->stage.itr)
	{
			/*--- Stop Stage -----------------------------*/
		case ciStop:
			VE_SetdataToVEreg_Stop(_ipdrv, _motor);
			break;

			/*--- Bootstrap Stage ------------------------*/
		case ciBootstrap:
			VE_SetdataToVEreg_Bootstrap(_ipdrv, _motor);
			break;

			/*--- Position Stage (Current type) ----------*/
		case ciInitposition_i:
			VE_SetdataToVEreg_Initposition_i(_ipdrv, _motor);
			break;

			/*--- Position Stage (Voltage type) ----------*/
		case ciInitposition_v:
			VE_SetdataToVEreg_Initposition_v(_ipdrv, _motor);
			break;

			/*--- Force Stage (Current type) -------------*/
		case ciForce_i:
			VE_SetdataToVEreg_Force_i(_ipdrv, _motor);
			break;

			/*--- Force Stage (Voltage type) -------------*/
		case ciForce_v:
			VE_SetdataToVEreg_Force_v(_ipdrv, _motor);
			break;

			/*--- ChangeUp Stage -------------------------*/
		case ciChange_up:
			VE_SetdataToVEreg_Change_up(_ipdrv, _motor);
			break;

			/*--- Steady_A Stage ---------------------------*/
		case ciSteady_A:
			VE_SetdataToVEreg_Steady_A(_ipdrv, _motor);
			break;

			/*--- Emergency Stage ------------------------*/
		case ciEmergency:
			VE_SetdataToVEreg_Emergency(_ipdrv, _motor);
			break;
			/*--- Fail-safe ------------------------*/
		default:
			VE_SetdataToVEreg_Stop(_ipdrv, _motor);
			break;
	}

#if defined(__USE_DAC)
	UiOutDataStartByVE(2);									/* Dac output for debug */
#endif /* __USE_DAC */

	VE_Start(_ipdrv);

}
#endif /* defined(__CONTROL_MOTOR_CH2) && defined(__USE_VE_CH2) */

/**
 ********************************************************************************************
 * @brief		Vector Control Interruption for ch0. (All software control.)
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)
void INT_VectorControlBySoft_CH0(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{

	/***** Get data from ADC register *****/
	D_GetMotorCurrentPowerVolt(_motor, _ipdrv);
	
	/***** Check over current *****/
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}
	/***** Check over voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH0 / cV_MAX_CH0)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH0 / cV_MAX_CH0) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Input phase transformation *****/
		D_InputTransformation(_motor);

		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH0)
			/***** Encoder *****/
		H_Encoder(_motor, TSB_EN0);
#endif /* __USE_ENCODER_CH0 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH0)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}

		/***** Check Current detection error  *****/
	 	D_Check_DetectCurrentError(_motor);
		DEBUG_IDET_ERROR(_motor);

		/***** Current control  *****/
		D_Control_Current(_motor);

		/***** Output coordinate axis transformation  *****/
		D_OutputTransformation(_motor);

		/***** Trigger generation  *****/
		D_CalTrgTiming(_motor);
	}

	/***** Output control  *****/
	PMD_SetTriggerSelect(_ipdrv, _motor->drv.Sector1);
	PMD_RegDataSet(_ipdrv, (uint16_t)_motor->drv.DutyU, (uint16_t)_motor->drv.DutyV, (uint16_t)_motor->drv.DutyW, (q15_t)_motor->drv.AdTrg0, (q15_t)_motor->drv.AdTrg1);

#if MOTOR_OUTPUT == 1
	D_DecisionOutputMode(_motor, _ipdrv);
#endif

#if defined(__USE_DAC)
	UiOutDataStart(0);									/* Dac output for debug */
#endif /* __USE_DAC */

}
#endif /* defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0) */


/**
 ********************************************************************************************
 * @brief		Vector Control Interruption for ch1. (All software control.)
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)
void INT_VectorControlBySoft_CH1(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	/***** Get data from ADC register *****/
	D_GetMotorCurrentPowerVolt(_motor, _ipdrv);
	
	/***** Check over current *****/
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}
	/***** Check over voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH1 / cV_MAX_CH1)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH1 / cV_MAX_CH1) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Input phase transformation *****/
		D_InputTransformation(_motor);

		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH1)
			/***** Encoder *****/
		H_Encoder(_motor, TSB_EN1);
#endif /* __USE_ENCODER_CH1 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH1)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}

		/***** Check Current detection error  *****/
	 	D_Check_DetectCurrentError(_motor);
		DEBUG_IDET_ERROR(_motor);

		/***** Current control  *****/
		D_Control_Current(_motor);

		/***** Output coordinate axis transformation  *****/
		D_OutputTransformation(_motor);

		/***** Trigger generation  *****/
		D_CalTrgTiming(_motor);
	}

	/***** Output control  *****/
	PMD_SetTriggerSelect(_ipdrv, _motor->drv.Sector1);
	PMD_RegDataSet(_ipdrv, (uint16_t)_motor->drv.DutyU, (uint16_t)_motor->drv.DutyV, (uint16_t)_motor->drv.DutyW, (q15_t)_motor->drv.AdTrg0, (q15_t)_motor->drv.AdTrg1);

#if MOTOR_OUTPUT == 1
	D_DecisionOutputMode(_motor, _ipdrv);
#endif

#if defined(__USE_DAC)
	UiOutDataStart(1);									/* Dac output for debug */
#endif /* __USE_DAC */

}
#endif /* defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1) */

/**
 ********************************************************************************************
 * @brief		Vector Control Interruption for ch2. (All software control.)
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	const ipdrv_t* const 	_ipdrv	.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)
void INT_VectorControlBySoft_CH2(vector_t* const _motor, const ipdrv_t* const _ipdrv)
{
	/***** Get data from ADC register *****/
	D_GetMotorCurrentPowerVolt(_motor, _ipdrv);
	
	/***** Check over current *****/
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}
	/***** Check over voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH2 / cV_MAX_CH2)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH2 / cV_MAX_CH2) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Input phase transformation *****/
		D_InputTransformation(_motor);

		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH2)
			/***** Encoder *****/
		H_Encoder(_motor, TSB_EN2);
#endif /* __USE_ENCODER_CH2 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH2)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}

		/***** Check Current detection error  *****/
	 	D_Check_DetectCurrentError(_motor);
		DEBUG_IDET_ERROR(_motor);

		/***** Current control  *****/
		D_Control_Current(_motor);

		/***** Output coordinate axis transformation  *****/
		D_OutputTransformation(_motor);

		/***** Trigger generation  *****/
		D_CalTrgTiming(_motor);
	}

	/***** Output control  *****/
	PMD_SetTriggerSelect(_ipdrv, _motor->drv.Sector1);
	PMD_RegDataSet(_ipdrv, (uint16_t)_motor->drv.DutyU, (uint16_t)_motor->drv.DutyV, (uint16_t)_motor->drv.DutyW, (q15_t)_motor->drv.AdTrg0, (q15_t)_motor->drv.AdTrg1);

#if MOTOR_OUTPUT == 1
	D_DecisionOutputMode(_motor, _ipdrv);
#endif

#if defined(__USE_DAC)
	UiOutDataStart(2);									/* Dac output for debug */
#endif /* __USE_DAC */

}
#endif /* defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2) */


/*********************************** END OF FILE ******************************/
