/**
 ****************************************************************************
 * @file:	 mcuip_drv.c
 * @brief:	 MCU IP driver for A-VE Plus
 * @version:	 V2.2
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include "ipdefine.h"
#include "mcuip_drv.h"


/**
 *********************************************************************************************
 * @brief		Get EMGstatus
 *
 * @param		const ipdrv_t* const _ipdrv	Select the IP table.
 *
 * @return		emg_status_e				EMG status
 *********************************************************************************************
 */
emg_status_e	PMD_GetEMG_Status(const ipdrv_t* const _ipdrv)
{
	TSB_PMD_TypeDef* const PMDx = _ipdrv->PMDx;

	return ((emg_status_e)(*((__I uint32_t*)BITBAND_PERI(&PMDx->EMGSTA, B_EMGST))));

}

/**
 *********************************************************************************************
 * @brief		Release EMG protection.
 *
 * @param		const ipdrv_t* const _ipdrv	Select the IP table.
 *
 * @return		none
 *********************************************************************************************
 */
void	PMD_ReleaseEMG_Protection(const ipdrv_t* const _ipdrv)
{
	TSB_PMD_TypeDef* const PMDx = _ipdrv->PMDx;
	emg_status_e	emgst;

	emgst = PMD_GetEMG_Status(_ipdrv);

	if (emgst == cEMGProtected)
	{
		if ((*((__I uint32_t*)BITBAND_PERI(&PMDx->EMGSTA, B_EMGI))) == 1)
		{
			if (PMDx == TSB_PMD2)
			{
			#if defined(__CONTROL_MOTOR_CH2)
				#if !defined(__USE_VE_CH2)
				(*((__O uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGRS))) = 1;
				#endif
			#endif
			}
			else if (PMDx == TSB_PMD1)
			{
			#if defined(__CONTROL_MOTOR_CH1)
				#if !defined(__USE_VE_CH1)
				(*((__O uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGRS))) = 1;
				#endif
			#endif
			}
			else
			{
			#if defined(__CONTROL_MOTOR_CH0)
				#if !defined(__USE_VE_CH0)
				(*((__O uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGRS))) = 1;
				#else
				TSB_VE_TypeDef* const VEx = _ipdrv->VEx;
				VEx->EMGRS = 1;						/*Release EMG protection */
				#endif
			#endif
			}
		}
	}

}

/**
 *********************************************************************************************
 * @brief		PMD_RegDataSet
 *
 * @param		const ipdrv_t* const _ipdrv			Select the IP table.
 * @param		 uint16_t  duty_u					U-phase PWM duty ratio
 * @param		 uint16_t  duty_v					V-phase PWM duty ratio
 * @param		 uint16_t  duty_w					W-phase PWM duty ratio
 * @param		 q15_t  adtrg0						AD trigger timing 0 position
 * @param		 q15_t  adtrg1						AD trigger timing 1 position
 *
 * @return		none	
 *********************************************************************************************
 */
void PMD_RegDataSet(const ipdrv_t* const _ipdrv, uint16_t duty_u, uint16_t duty_v, uint16_t duty_w, q15_t adtrg0, q15_t adtrg1)
{
	TSB_PMD_TypeDef* const PMDx = _ipdrv->PMDx;
	int32_t trgtmp0, trgtmp1;

	/* TRGCMPx = 0 to 0x7FFF */
	trgtmp0 = ((int32_t)adtrg0 + 32768) / 2;
	if (trgtmp0 >= 0x8000) trgtmp0 = 0x7fff;
	trgtmp1 = ((int32_t)adtrg1 + 32768) / 2;
	if (trgtmp1 >= 0x8000) trgtmp1 = 0x7fff;

	PMDx->TRGCMP0 = trgtmp0;
	PMDx->TRGCMP1 = trgtmp1;

	/* When <CMPx> >= 0x8000, the duty ratio is 100%.*/
	PMDx->CMPU = duty_u;
	PMDx->CMPV = duty_v;
	PMDx->CMPW = duty_w;
}



/**
 ********************************************************************************************
 * @brief		Set OutputMode Register.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param[in]	uint32_t				_mdout	.
 *
 * @return		none
 ********************************************************************************************
 */
void PMD_SetOutputMode(const ipdrv_t* const _ipdrv, uint32_t _mdout)
{
	TSB_PMD_TypeDef* const PMDx = _ipdrv->PMDx;

	PMDx->MDOUT = _mdout;
	
}


/**
 ********************************************************************************************
 * @brief		Set Trigger select Register.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param[in]	uint32_t				_trgsel	.
 *
 * @return		none
 ********************************************************************************************
 */
void PMD_SetTriggerSelect(const ipdrv_t* const _ipdrv, uint32_t _trgsel)
{
	TSB_PMD_TypeDef* const PMDx = _ipdrv->PMDx;

	PMDx->TRGSEL = _trgsel;
	
}


/**
 *********************************************************************************************
 * @brief		VE start
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_Start(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->CPURUNTRG = VE_START;

}

/**
 *********************************************************************************************
 * @brief		VE_GetTask
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		uint32_t						Task No.
 *********************************************************************************************
 */
uint32_t	VE_GetTask(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	return((VEx->SCHTASKRUN >> 1U) & 0x0000000f);
}

/**
 *********************************************************************************************
 * @brief		Get Phase Current from VE register.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		q15_t*					_ia		U-phase current value store address.
 * @param		q15_t*					_ib		V-phase current value store address.
 * @param		q15_t*					_ic		W-phase current value store address.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_GetPhaseCurrent(const ipdrv_t* const _ipdrv, q15_t* _ia, q15_t* _ib, q15_t* _ic)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	*_ia = VEx->IA >> 16;
	*_ib = VEx->IB >> 16;
	*_ic = VEx->IC >> 16;
}

/**
 *********************************************************************************************
 * @brief		Get current adc data from VE register.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		uint32_t*				_adc_ia	U-phase current ad value store address.
 * @param		uint32_t*				_adc_ib	V-phase current ad value store address.
 * @param		uint32_t*				_adc_ic	W-phase current ad value store address.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_GetCurrentAdcData(const ipdrv_t* const _ipdrv, uint32_t* _adc_ia, uint32_t* _adc_ib, uint32_t* _adc_ic)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	*_adc_ia = VEx->IAADC;
	*_adc_ib = VEx->IBADC;
	*_adc_ic = VEx->ICADC;

}

/**
 *********************************************************************************************
 * @brief		Get data from VE registers.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_GetdataFromVEreg(const ipdrv_t* const _ipdrv, vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	_motor->drv.Vdc = VEx->VDC;									/* Read Vdc */
	_motor->drv.Vd.word = VEx->VD;								/* Read Vd */
	_motor->drv.Vq.word = VEx->VQ;								/* Read Vd */

	if (_motor->drv.idetect_error == 0)
	{
		_motor->drv.Id = (q15_t)(VEx->ID >> 16);				/* Read Id */
		_motor->drv.Iq = (q15_t)(VEx->IQ >> 16);				/* Read Iq */
	}
	else
	{
		VEx->ID = (q31_t)_motor->drv.Id << 16;					/* Set Id */
		VEx->IQ = (q31_t)_motor->drv.Iq << 16;					/* Set Iq */
	}
}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_DutyU
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		uint32_t						U Phase duty.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyU(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	return(VEx->CMPU);
}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_DutyV
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		uint32_t						V Phase duty.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyV(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	return(VEx->CMPV);
}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_DutyW
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		uint32_t						W Phase duty.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyW(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	return(VEx->CMPW);
}

/**
 *********************************************************************************************
 * @brief		Get medium duty from VE registers.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		uint32_t						Medium duty value.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyMed(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;
	uint32_t	a,b,c,ret;

	a = VEx->CMPU;
	b = VEx->CMPV;
	c = VEx->CMPW;

	if (a >= b)
	{
		ret = (c < b) ? b : (c > a) ? a : c;
	}
	else
	{
		ret = (c < a) ? a : (c > b) ? b : c;
	}

	return ret;

}

/**
 *********************************************************************************************
 * @brief		Get medium duty from VE registers.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		uint32_t						Medium duty value.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyMin(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;
	uint32_t	a,b,c,ret;

	a = VEx->CMPU;
	b = VEx->CMPV;
	c = VEx->CMPW;

	if (a >= b)
	{
		ret = (c < b) ? c : b;
	}
	else
	{
		ret = (c < a) ? c : a;
	}

	return ret;

}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_Modulation
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		int								PWM modulation 0:3Phase, 1:2Phase
 *********************************************************************************************
 */
int	VE_GetPWM_Modulation(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;
	int	ret = 0;

	if ((VEx->FMODE & (1 << B_C2PEN)) != 0)
	{
		ret = 1;
	}

	return(ret);

}

/**
 *********************************************************************************************
 * @brief		VE_GetSector
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		uint32_t						Sector
 *********************************************************************************************
 */
uint32_t	VE_GetSector(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	return(VEx->SECTOR);
}

/**
 *********************************************************************************************
 * @brief		VE_GetShiftPWMState
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		int								PWM Shift state 0:Normal PWM, 1;Shift PWM
 *********************************************************************************************
 */
int	VE_GetShiftPWMState(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;
	int	ret = 0;

	if ((VEx->MCTLF & (1 << B_LAVF)) != 0)
	{
		ret = 1;
	}

	return(ret);
}

/**
 *********************************************************************************************
 * @brief		VE_GetOutputMode
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		int	
 *********************************************************************************************
 */
int	VE_GetOutputMode(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	return(((VEx->MODE >> 2U) & 0x00000003));
}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Stop stage.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Stop(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_OUT_CON);						/* VE start from output control */
	VEx->ACTSCH = SET_ACTSCH(SCH_9);							/* Schedule 9 */

	VEx->MODE = VEMODE_STOP;									/* Output OFF */

	VEx->ID = 0;
	VEx->IQ = 0;
	VEx->VD = 0;
	VEx->VQ = 0;
	VEx->VDIH = 0;												/* Clear Current PI Control Integral Term Registers */
	VEx->VDILH = 0;
	VEx->VQIH = 0;
	VEx->VQILH = 0;
	VEx->IDREF = 0;
	VEx->IQREF = 0;
	VEx->THETA = 0;
	VEx->OMEGA = 0;
	VEx->SECTOR = 0;
	VEx->SECTORM = 0;
	VEx->MINPLS = 0;

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Bootstrap stage.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Bootstrap(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_SINCOS);						/* VE start from SIN/COS */
	VEx->ACTSCH = SET_ACTSCH(SCH_4);							/* Schedule 4 */

	VEx->MODE = VEMODE_BOOT;									/* Output ON */

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Initposition stage for current control.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Initposition_i(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_INITPOS;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Set Id */
	VEx->IQREF = _motor->drv.Iq_ref; 							/* Set Iq */
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}
}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Initposition stage for voltage control.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Initposition_v(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_SINCOS);						/* VE start from SIN/COS */
	VEx->ACTSCH = SET_ACTSCH(SCH_4);							/* Schedule 4 */

	VEx->MODE = VEMODE_INITPOS;									/* Output ON */

	VEx->VD = _motor->drv.Vd_out;								/* Vd */
	VEx->VQ = 0;
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Force stage for current control.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Force_i(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_FORCE;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Set Id */
	VEx->IQREF = _motor->drv.Iq_ref; 							/* Set Iq */
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Force stage for voltage control.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Force_v(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_SINCOS);						/* VE start from SIN/COS */
	VEx->ACTSCH = SET_ACTSCH(SCH_4);							/* Schedule 4 */

	VEx->MODE = VEMODE_FORCE;									/* Output ON */

	VEx->VD = _motor->drv.Vd_out;								/* Vd */
	VEx->VDIH = _motor->drv.Vd_out;								/* Vd Integral value initialization */
	VEx->VQ = 0; 												/* Vq = 0 */
	VEx->THETA = _motor->drv.theta.half[1];						/* Theta set */
	VEx->OMEGA = _motor->drv.omega.half[1];						/* Omega set */
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Change_up stage.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Change_up(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_CHGUP;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Set Id */
	VEx->IQREF = _motor->drv.Iq_ref; 							/* Set Iq */
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Steady_A stage.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Steady_A(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_STEADY;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Id set */
	VEx->IQREF = _motor->drv.Iq_ref;							/* Iq set */
	VEx->THETA = _motor->drv.theta.half[1];						/* Theta set */
	VEx->OMEGA = _motor->drv.omega.half[1];						/* Omega set */
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Emergency stage.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Emergency(const ipdrv_t* const _ipdrv, const vector_t* const _motor)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->TASKAPP = SET_TAPP(TAPP_OUT_CON);						/* VE start from output control */
	VEx->ACTSCH = SET_ACTSCH(SCH_9);							/* Schedule 9 */

	VEx->MODE = VEMODE_EMG;										/* Output OFF */

}

/**
 *********************************************************************************************
 * @brief		Set zero current data to VE register.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		uint32_t				_z_ia	U Phase value.
 * @param		uint32_t				_z_ib	V Phase value.
 * @param		uint32_t				_z_ic	W Phase value.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetZeroCurrentData(const ipdrv_t* const _ipdrv, uint32_t _z_ia, uint32_t _z_ib, uint32_t _z_ic)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->IAO = _z_ia;
	VEx->IBO = _z_ib;
	VEx->ICO = _z_ic;

}

/**
 *********************************************************************************************
 * @brief		Set VEVDC reg.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		q15_t					_dat	Power supply voltage(Q15)
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetVDCreg(const ipdrv_t* const _ipdrv, q15_t _dat)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	VEx->VDC = _dat;
}

/**
 *********************************************************************************************
 * @brief		Set shift pwm_mode.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		uint8_t					_dat	Shift pwm mode.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetSpwmMode(const ipdrv_t* const _ipdrv, uint8_t _dat)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	(*((__IO uint32_t*)BITBAND_PERI(&VEx->FMODE, B_SPWMEN))) = _dat;
}

/**
 *********************************************************************************************
 * @brief		Set modulation type.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 * @param		uint8_t					_dat	modulation type.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetModulType(const ipdrv_t* const _ipdrv, uint8_t _dat)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	(*((__IO uint32_t*)BITBAND_PERI(&VEx->FMODE, B_C2PEN))) = _dat;
}

/**
 *********************************************************************************************
 * @brief		Check conversion finishing flag.
 *
 * @param		const ipdrv_t* const 	_ipdrv	Select the IP table.
 *
 * @return		none
 *********************************************************************************************
 */
void	AD_CheckFinishedConversion(const ipdrv_t* const _ipdrv)
{
	TSB_AD_TypeDef* const ADx = _ipdrv->ADx;

	while ((*((__I uint32_t*)BITBAND_PERI(&ADx->ST, B_PMDF))) == SET)
	{
	}
}

/**
 *********************************************************************************************
 * @brief		Get Power Supply Voltage.
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
void	AD_GetVdcData(vector_t* const _motor)
{
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
	/* Power supply voltage is positive. 1bit right shift.(A sign is removed.) */
	_motor->drv.Vdc = _motor->drv.Vdc_adc = (q15_t)((*_motor->drv.ADxREG3 & 0xFFF0) >> 1);
#endif /* (defined(__CONTORL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */
}

/**
 *********************************************************************************************
 * @brief		Acquire AD at the position in the first half of the PWM cycle.
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
void	AD_GetIdcFirstHalf(vector_t* const _motor)
{
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
	_motor->drv.Idc1_adc = (q15_t)(*_motor->drv.ADxREG1 & 0xFFF0);
	_motor->drv.Idc2_adc = (q15_t)(*_motor->drv.ADxREG0 & 0xFFF0);
#endif /* (defined(__CONTORL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */
}

/**
 *********************************************************************************************
 * @brief		Acquire AD at the position in the second half of the PWM cycle.
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
void	AD_GetIdcSecondHalf(vector_t* const _motor)
{
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
	_motor->drv.Idc1_adc = (q15_t)(*_motor->drv.ADxREG0 & 0xFFF0);
	_motor->drv.Idc2_adc = (q15_t)(*_motor->drv.ADxREG1 & 0xFFF0);
#endif /* (defined(__CONTORL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */
}

/**
 *********************************************************************************************
 * @brief		PMD IP Initialize
 *
 * @param		TSB_PMD_TypeDef* const	PMDx		Select the PMD channel.
 * @param		PMD_InitTypeDef* const	_initdata	The structure containing basic PMD configuration.
 *
 * @return		none
 *********************************************************************************************
 */
void IP_PMD_init(TSB_PMD_TypeDef* const PMDx, PMD_InitTypeDef* const _initdata)
{
	if (PMDx == TSB_PMD2)
	{
		CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_PMD2, ENABLE);
	}
	else if (PMDx == TSB_PMD1)
	{
		CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_PMD1, ENABLE);
	}
	else
	{
		CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_PMD0, ENABLE);
	}

	PMDx->MODESEL = WBUF_BUS;

	/***** active setting *****/
	PMDx->MDPOT = cMDPOT_INI;
	if (_initdata->poll == 1)
	{
		PMDx->MDPOT |= (1U << B_POLL);
	}
	else
	{
		PMDx->MDPOT &= ~(1U << B_POLL);
	}
	if (_initdata->polh == 1)
	{
		PMDx->MDPOT |= (1U << B_POLH);
	}
	else
	{
		PMDx->MDPOT &= ~(1U << B_POLH);
	}

	PMDx->MDOUT = cMDOUT_OFF;									/* Output off */
	PMDx->CMPU = 0;
	PMDx->CMPV = 0;
	PMDx->CMPW = 0;

	/***** EMG setting *****/
#if (1)
	PMDx->EMGCR = cEMG_ENA; 									/* EMG Enable */
	PMDx->EMGCR = cEMG_Release; 								/* EMG Release */
#else
	PMDx->EMGREL = 0x5A;										/* Disable code */
	PMDx->EMGREL = 0xA5;
	PMDx->EMGCR = cEMG_DIS; 									/* EMG Disable */
#endif

	/* OVV disable set */
	PMDx->EMGREL = 0x5A;										/* Disable code */
	PMDx->EMGREL = 0xA5;
	PMDx->OVVCR = cOVV_DIS;

	PMDx->PORTMD = cPORTMD_INI;									/* Tool break */
	PMDx->MDCR = cMDCR_INI;

	PMDx->RATE = _initdata->pwmrate;							/* PWM rate */
	PMDx->DTR = _initdata->deadtime; 							/* Dead timewr set */
	PMDx->MDEN = 0x01;											/* PMD enable (Set after set MDOUT,MDPOT) */

	/***** port setting *****/
	if (PMDx == TSB_PMD2)
	{
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTU, ENABLE);
        TSB_PU->FR6 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD2 PORT output Enable */
		TSB_PU->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD2 PORT output Enable */

		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGEN))) == 1)	/* EMG enable? */
		{
			TSB_PU->FR6 |= BIT8(0, 1, 0, 0, 0, 0, 0, 0);		/* PMD2 PORT Enable */
			TSB_PU->IE  |= BIT8(0, 1, 0, 0, 0, 0, 0, 0); 		/* PMD2 PORT Enable */
		}

		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->OVVCR, B_OVVEN))) == 1)	/* OVV enable? */
		{
			TSB_PU->FR6 |= BIT8(1, 0, 0, 0, 0, 0, 0, 0);		/* PMD2 PORT Enable */
			TSB_PU->IE  |= BIT8(1, 0, 0, 0, 0, 0, 0, 0); 		/* PMD2 PORT Enable */
		}

	#if defined(__MOTOR_PMDDBG)	/* for debug */
	/* PMD DBG for CH2 */
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTA, ENABLE);
		TSB_PA_CR_PA2C = SET;
		TSB_PA_FR6_PA2F6 = SET;

	//	PMDx->DBGOUTCR = ((0x3FU << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger timing monitor */
	//	PMDx->DBGOUTCR = ((0x03U << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger 0,1 timing monitor for 1shunt */
	//	PMDx->DBGOUTCR = ((1U << 8) | (2U << 1)  | (1U << 0));	/* INTPWMx PWM Interrupt monitor */
	//	PMDx->DBGOUTCR = ((1U << 3) | (2U << 1)  | (1U << 0));	/* INTADxPA  Interrupt monitor */
	//	PMDx->DBGOUTCR = DBGOUT_ENA_VE;		/* VE Task timing monitor */
		PMDx->DBGOUTCR = DBGOUT_ENA_ADC;	//((0x03U << 16) | (1U << 0));	/* ADC convert timing monitor */
	#endif

	}
	else if (PMDx == TSB_PMD1)
	{
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTE, ENABLE);
        TSB_PE->FR6 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD1 PORT output Enable */
		TSB_PE->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD1 PORT output Enable */

		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGEN))) == 1)	/* EMG enable? */
		{
			TSB_PE->FR6 |= BIT8(0, 1, 0, 0, 0, 0, 0, 0);		/* PMD1 PORT Enable */
			TSB_PE->IE  |= BIT8(0, 1, 0, 0, 0, 0, 0, 0); 		/* PMD1 PORT Enable */
		}

		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->OVVCR, B_OVVEN))) == 1)	/* OVV enable? */
		{
			TSB_PE->FR6 |= BIT8(1, 0, 0, 0, 0, 0, 0, 0);		/* PMD1 PORT Enable */
			TSB_PE->IE  |= BIT8(1, 0, 0, 0, 0, 0, 0, 0); 		/* PMD1 PORT Enable */
		}

	#if defined(__MOTOR_PMDDBG)	/* for debug */
	/* PMD DBG for CH1 */
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTC, ENABLE);
		TSB_PC_CR_PC3C = SET;
		TSB_PC_FR7_PC3F7 = SET;

	//	PMDx->DBGOUTCR = ((0x3FU << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger timing monitor */
	//	PMDx->DBGOUTCR = ((0x03U << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger 0,1 timing monitor for 1shunt */
	//	PMDx->DBGOUTCR = ((1U << 8) | (2U << 1)  | (1U << 0));	/* INTPWMx PWM Interrupt monitor */
	//	PMDx->DBGOUTCR = ((1U << 3) | (2U << 1)  | (1U << 0));	/* INTADxPA  Interrupt monitor */
	//	PMDx->DBGOUTCR = DBGOUT_ENA_VE;		/* VE Task timing monitor */
		PMDx->DBGOUTCR = DBGOUT_ENA_ADC;	//((0x03U << 16) | (1U << 0));	/* ADC convert timing monitor */
	#endif

	}
	else if (PMDx == TSB_PMD0)
	{
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTB, ENABLE);
		TSB_PB->FR4 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD0 PORT output Enable */
		TSB_PB->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD0 PORT output Enable */
		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGEN))) == 1)	/* EMG enable? */
		{
			TSB_PB->FR4 |= BIT8(0, 1, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
			TSB_PB->IE  |= BIT8(0, 1, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
		}

		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->OVVCR, B_OVVEN))) == 1)	/* OVV enable? */
		{
			TSB_PB->FR4 |= BIT8(1, 0, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
			TSB_PB->IE  |= BIT8(1, 0, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
		}

	#if defined(__MOTOR_PMDDBG)	/* for debug */
	/* PMD DBG for CH0 */
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTC, ENABLE);
		TSB_PC_CR_PC2C = SET;
		TSB_PC_FR7_PC2F7 = SET;

	//	PMDx->DBGOUTCR = ((0x3FU << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger timing monitor */
	//	PMDx->DBGOUTCR = ((0x03U << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger 0,1 timing monitor for 1shunt */
	//	PMDx->DBGOUTCR = ((1U << 8) | (2U << 1)  | (1U << 0));	/* INTPWMx PWM Interrupt monitor */
	//	PMDx->DBGOUTCR = ((1U << 3) | (2U << 1)  | (1U << 0));	/* INTADxPA  Interrupt monitor */
	//	PMDx->DBGOUTCR = DBGOUT_ENA_VE;		/* VE Task timing monitor */
		PMDx->DBGOUTCR = DBGOUT_ENA_ADC;	//((0x03U << 16) | (1U << 0));	/* ADC convert timing monitor */
	#endif

	}
	else
	{
		/* Do nothing */
	}

        #if defined(__MOTOR_DBGOUT_IDET)  /* for debug2020 */
        TSB_CG_FSYSMENA_IPMENA03 = SET;
        TSB_PD_CR_PD0C = SET;
        #endif
        
	/***** Release EMG protection *****/
	if ((*((__I uint32_t*)BITBAND_PERI(&PMDx->EMGSTA, B_EMGI))) == 1)
	{
		(*((__O uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGRS))) = 1;
	}


	/***** Triger setting *****/
	PMDx->TRGCMP0 = 0; 											/* TRG position clear */
	PMDx->TRGCMP1 = 0;
	PMDx->TRGCMP2 = 0;
	PMDx->TRGCMP3 = 0;

	if (_initdata->shunt == c3shunt)
	{
		PMDx->TRGCR = cTRG_3SHUNT;
		PMDx->TRGMD = cTRGMD_3SHUNT;
	}
	else if (_initdata->shunt == c1shunt)
	{
		PMDx->TRGCR = cTRG_1SHUNT;
		PMDx->TRGMD = cTRGMD_1SHUNT;
	}

	PMDx->TRGSEL = 0;											/* TRG No = Sector = 0 */

	PMDx->TRGSYNCR = cTRGSYNCR_INI;

	PMDx->MODESEL = _initdata->busmode;
	
}

/**
 *********************************************************************************************
 * @brief		IP_VE_init
 *
 * @param		TSB_VE_TypeDef* const	VEx			Select the VE channel.
 * @param		VE_InitTypeDef* const	_initdata	The structure containing basic VE configuration
 *
 * @return		none
 *********************************************************************************************
 */
void IP_VE_init(TSB_VE_TypeDef* const VEx, VE_InitTypeDef* const _initdata)
{
	CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_A_VE_P, ENABLE);

	/*********************** VE Control Register ********************************/
	VEx->EN = 0x01; 											/* VE Enable */

	VEx->TASKAPP = SET_TAPP(TAPP_OUT_CON);						/* VE start from output control */
	VEx->ACTSCH = SET_ACTSCH(SCH_9); 							/* Schedule 9 */

	VEx->REPTIME = SET_REPTIME(_initdata->reptime);
	VEx->TRGMODE = SET_TRGMODE(_initdata->trgmode);

	VEx->ERRINTEN = SET_ERRINTEN(cVERREN_INI);

	VEx->TMPREG0 = 0;											/* Temporary register clear */
	VEx->TMPREG1 = 0;
	VEx->TMPREG2 = 0;
	VEx->TMPREG3 = 0;
	VEx->TMPREG4 = 0;
	VEx->TMPREG5 = 0;

	VEx->TADC = (uint16_t)(cTADC * cIMCLK_FRQ * 1e6);			/* for 1shunt shift pwm.  (AD convert time) */

	/*********************** VE Register for Channel 1 **************************/
	VEx->MODE = VEMODE_STOP;

	if (_initdata->shunt == c3shunt)
	{
		VEx->FMODE = cFMODE_INI_3SHUNT;
	}
	else
	{
		VEx->FMODE = cFMODE_INI_1SHUNT;
	}

	VEx->TPWM = _initdata->tpwm;

	VEx->OMEGA = 0;
	VEx->THETA = 0;

	VEx->COS = 0;
	VEx->SIN = 0;
	VEx->COSM = 0;
	VEx->SINM = 0;

	VEx->IDREF = 0;
	VEx->IQREF = 0;

	VEx->VD = 0;
	VEx->VQ = 0;

	VEx->CIDKI = _initdata->idki;
	VEx->CIDKP = _initdata->idkp;
	VEx->CIDKG = _initdata->idkg;
	VEx->CIQKI = _initdata->iqki;
	VEx->CIQKP = _initdata->iqkp;
	VEx->CIQKG = _initdata->iqkg;

	VEx->VDIH = 0;
	VEx->VDILH = 0;
	VEx->VQIH = 0;
	VEx->VQILH = 0;

	VEx->MCTLF = 0x00;

	VEx->FPWMCHG = 0x7fff;										/* Hard control off */

	VEx->MINPLS = 0;

	VEx->SECTOR = 0;
	VEx->SECTORM = 0;

	VEx->IAO = _initdata->zerooffset;
	VEx->IBO = _initdata->zerooffset;
	VEx->ICO = _initdata->zerooffset;

	VEx->IAADC = 0;
	VEx->IBADC = 0;
	VEx->ICADC = 0;

	VEx->VDC = 0;
	VEx->ID = 0;
	VEx->IQ = 0;

	VEx->CMPU = 0;
	VEx->CMPV = 0;
	VEx->CMPW = 0;

	VEx->OUTCR = 0x0000;										/* Output UP/DOWN = OFF/OFF */

	VEx->TRGCRC = 0;

	VEx->TRGCMP0 = 0;
	VEx->TRGCMP1 = 0;

	VEx->TRGSEL = 0;

	VEx->EMGRS = 0;

	VEx->CPURUNTRG = VE_START;

}

/**
 *********************************************************************************************
 * @brief		ADC Initial set
 *
 * @param		TSB_AD_TypeDef* const	ADx			Select the ADC channel.
 * @param		AD_InitTypeDef* const	_initdata	The structure containing basic ADC configuration
 *
 * @return		none
 *********************************************************************************************
 */
void IP_ADC_init(TSB_AD_TypeDef* const ADx, AD_InitTypeDef* const _initdata)
{
	uint32_t data32;

	/***** PMD TRG Start mode set *****/

	CG_SetFcSupplyPeriph((CG_FC_PERIPH_DNF_A | CG_FC_PERIPH_DNF_B | CG_FC_PERIPH_DNF_C), ENABLE);
	CG_SetAdcClkSupply(ADx, ENABLE);
	if (_initdata->pmd_ch == cPMD0)
	{
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTL, ENABLE);
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTM, ENABLE);
		CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_ADC_A, ENABLE);
	}
	else if (_initdata->pmd_ch == cPMD1)
	{
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTK, ENABLE);
		CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_ADC_B, ENABLE);
	}
	else if (_initdata->pmd_ch == cPMD2)
	{
		CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTJ, ENABLE);
		CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_ADC_C, ENABLE);
	}
	else{
		/* Do nothing */
	}

	if (_initdata->shunt == c3shunt)
	{
		/* ADx Program set */
		if (_initdata->pmd_ch == cPMD0)
		{
			ADx->PSEL0  = TRG_ENABLE(PMD_PROG0);		/* PMD0 TRG0:BProg0 at Sector0	3-shunt */
			ADx->PSEL1  = TRG_ENABLE(PMD_PROG1);		/* PMD0 TRG1:BProg1 at Sector1	3-shunt */
			ADx->PSEL2  = TRG_ENABLE(PMD_PROG2);		/* PMD0 TRG2:BProg2 at Sector2	3-shunt */
			ADx->PSEL3  = TRG_ENABLE(PMD_PROG3);		/* PMD0 TRG3:BProg3 at Sector3	3-shunt */
			ADx->PSEL4  = TRG_ENABLE(PMD_PROG4);		/* PMD0 TRG4:BProg4 at Sector4	3-shunt */
			ADx->PSEL5  = TRG_ENABLE(PMD_PROG5);		/* PMD0 TRG5:BProg5 at Sector5	3-shunt */
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD1 TRG0:BProg0 at Sector0	3-shunt */
			ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD1 TRG1:BProg1 at Sector1	3-shunt */
			ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD1 TRG2:BProg2 at Sector2	3-shunt */
			ADx->PSEL9  = TRG_ENABLE(PMD_PROG3);		/* PMD1 TRG3:BProg3 at Sector3	3-shunt */
			ADx->PSEL10 = TRG_ENABLE(PMD_PROG4);		/* PMD1 TRG4:BProg4 at Sector4	3-shunt */
			ADx->PSEL11 = TRG_ENABLE(PMD_PROG5);		/* PMD1 TRG5:BProg5 at Sector5	3-shunt */
		}
		else if (_initdata->pmd_ch == cPMD2)
		{
			ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD2 TRG0:BProg0 at Sector0	3-shunt */
			ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD2 TRG1:BProg1 at Sector1	3-shunt */
			ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD2 TRG2:BProg2 at Sector2	3-shunt */
			ADx->PSEL9  = TRG_ENABLE(PMD_PROG3);		/* PMD2 TRG3:BProg3 at Sector3	3-shunt */
			ADx->PSEL10 = TRG_ENABLE(PMD_PROG4);		/* PMD2 TRG4:BProg4 at Sector4	3-shunt */
			ADx->PSEL11 = TRG_ENABLE(PMD_PROG5);		/* PMD2 TRG5:BProg5 at Sector5	3-shunt */
		}
		else{
			/* Do nothing */
		}
		
		if (_initdata->pmd_ch == cPMD0)
		{
			#if !defined(__USE_VE_CH0)
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD0_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-5. */
			ADx->PREGS = cPMD0_PREGS_3SHUNT;
			#endif /* !defined(__USE_VE_CH0) */
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			#if !defined(__USE_VE_CH1)
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD1_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-5. */
			ADx->PREGS = cPMD1_PREGS_3SHUNT;
			#endif /* !defined(__USE_VE_CH1) */
		}
		else if (_initdata->pmd_ch == cPMD2)
		{
			#if !defined(__USE_VE_CH2)
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD2_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD2_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD2_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD2_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-5. */
			ADx->PREGS = cPMD2_PREGS_3SHUNT;
			#endif /* !defined(__USE_VE_CH1) */
		}
		else{
			/* Do nothing */
		}

		if (_initdata->pints == cPINTS_A)
		{
			ADx->PINTS0 = PMD_INTADPDA;					/* End of all prog generate INTADA */
			ADx->PINTS1 = PMD_INTADPDA;
			ADx->PINTS2 = PMD_INTADPDA;
			ADx->PINTS3 = PMD_INTADPDA;
			ADx->PINTS4 = PMD_INTADPDA;
			ADx->PINTS5 = PMD_INTADPDA;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
		}
		else if (_initdata->pints == cPINTS_B)
		{
			ADx->PINTS0 = PMD_INTADPDB;					/* End of all prog generate INTADB */
			ADx->PINTS1 = PMD_INTADPDB;
			ADx->PINTS2 = PMD_INTADPDB;
			ADx->PINTS3 = PMD_INTADPDB;
			ADx->PINTS4 = PMD_INTADPDB;
			ADx->PINTS5 = PMD_INTADPDB;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
		}
		else{
			/* Do nothing */
		}

		/* ADx->PSET5,PSET0	REG0:V, REG1:W, REG2:U, REG3:Vdc */
		data32 = (((0x80 | (VE_PHASE_V    << 5U) | _initdata->ivch)) \
		        | ((0x80 | (VE_PHASE_W    << 5U) | _initdata->iwch)  << 8U) \
		        | ((0x80 | (VE_PHASE_U    << 5U) | _initdata->iuch)  << 16U) \
		        | ((0x80 | (VE_PHASE_NONE << 5U) | _initdata->vdcch) << 24U));
		ADx->PSET5 = data32;
		ADx->PSET0 = data32;

		/* ADx->PSET1,PSET2	REG0:W, REG1:U, REG2:V, REG3:Vdc */
		data32 = (((0x80 | (VE_PHASE_W    << 5U) | _initdata->iwch)) \
		        | ((0x80 | (VE_PHASE_U    << 5U) | _initdata->iuch)  << 8U) \
		        | ((0x80 | (VE_PHASE_V    << 5U) | _initdata->ivch)  << 16U) \
		        | ((0x80 | (VE_PHASE_NONE << 5U) | _initdata->vdcch) << 24U));
		ADx->PSET1 = data32;
		ADx->PSET2 = data32;

		/* ADx->PSET3,PSET4	REG0:U, REG1:V, REG2:W, REG3:Vdc */
		data32 = (((0x80 | (VE_PHASE_U    << 5U) | _initdata->iuch)) \
		        | ((0x80 | (VE_PHASE_V    << 5U) | _initdata->ivch)  << 8U) \
		        | ((0x80 | (VE_PHASE_W    << 5U) | _initdata->iwch)  << 16U) \
		        | ((0x80 | (VE_PHASE_NONE << 5U) | _initdata->vdcch) << 24U));
		ADx->PSET3	= data32;
		ADx->PSET4	= data32;

		ADx->PSET6	= 0;
		ADx->PSET7	= 0;

	}
	else if (_initdata->shunt == c1shunt)
	{
		/* ADx Program set */
		if (_initdata->pmd_ch == cPMD0)
		{
			ADx->PSEL0  = TRG_ENABLE(PMD_PROG0);		/* PMD0 TRG0:BProg0 1-shunt */
			ADx->PSEL1  = TRG_ENABLE(PMD_PROG1);		/* PMD0 TRG1:BProg1 1-shunt */
			ADx->PSEL2  = TRG_ENABLE(PMD_PROG2);		/* PMD0 TRG2:BProg2 */
			ADx->PSEL3  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG3:BProg3 */
			ADx->PSEL4  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG4:BProg4 */
			ADx->PSEL5  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG5:BProg5 */
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD1 TRG0:BProg0 1-shunt */
			ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD1 TRG1:BProg1 1-shunt */
			ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD1 TRG2:BProg2 */
			ADx->PSEL9  = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG3:BProg3 */
			ADx->PSEL10 = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG4:BProg4 */
			ADx->PSEL11 = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG5:BProg5 */
		}
		else if (_initdata->pmd_ch == cPMD2)
		{
			ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD2 TRG0:BProg0 1-shunt */
			ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD2 TRG1:BProg1 1-shunt */
			ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD2 TRG2:BProg2 */
			ADx->PSEL9  = TRG_DISABLE(PMD_PROG0);		/* PMD2 TRG3:BProg3 */
			ADx->PSEL10 = TRG_DISABLE(PMD_PROG0);		/* PMD2 TRG4:BProg4 */
			ADx->PSEL11 = TRG_DISABLE(PMD_PROG0);		/* PMD2 TRG5:BProg5 */
		}
		else{
			/* Do nothing */
		}

		if (_initdata->pmd_ch == cPMD0)
		{
			#if !defined(__USE_VE_CH0)
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD0_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-2. */
			ADx->PREGS = cPMD0_PREGS_1SHUNT;
			#endif /* !defined(__USE_VE_CH0) */
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			#if !defined(__USE_VE_CH1)
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD1_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-2. */
			ADx->PREGS = cPMD1_PREGS_1SHUNT;
			#endif /* !defined(__USE_VE_CH1) */
		}
		else if (_initdata->pmd_ch == cPMD2)
		{
			#if !defined(__USE_VE_CH2)
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD2_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD2_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD2_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD2_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-2. */
			ADx->PREGS = cPMD2_PREGS_1SHUNT;
			#endif /* !defined(__USE_VE_CH1) */
		}
		else{
			/* Do nothing */
		}

		if (_initdata->pints == cPINTS_A)
		{
			ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADA */
			ADx->PINTS1 = PMD_INTNONE;
			ADx->PINTS2 = PMD_INTADPDA;
			ADx->PINTS3 = PMD_INTNONE;
			ADx->PINTS4 = PMD_INTNONE;
			ADx->PINTS5 = PMD_INTNONE;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
		}
		else if (_initdata->pints == cPINTS_B)
		{
			ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADB */
			ADx->PINTS1 = PMD_INTNONE;
			ADx->PINTS2 = PMD_INTADPDB;
			ADx->PINTS3 = PMD_INTNONE;
			ADx->PINTS4 = PMD_INTNONE;
			ADx->PINTS5 = PMD_INTNONE;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
		}
		else{
			/* Do nothing */
		}

		/*ADx->PSET0	REG0:Idc----, REG1:-------, REG2:-------, REG3:------- */
		data32 = (0x80 | _initdata->idcch);
		ADx->PSET0 = data32;

		/*ADx->PSET1	REG0:-------, REG1:Idc----, REG2:-------, REG3:------- */
		data32 = (((0x80 | _initdata->idcch) << 8U));
		ADx->PSET1 = data32;

		/*ADx->PSET2	REG0:-------, REG1:-------, REG2:-------, REG3:Vdc---- */
		data32 = ((0x80 | _initdata->vdcch) << 24U);
		ADx->PSET2 = data32;

		ADx->PSET3 = 0;
		ADx->PSET4 = 0;
		ADx->PSET5 = 0;
		ADx->PSET6 = 0;
		ADx->PSET7 = 0;
	}

}


/**
 *********************************************************************************************
 * @brief		AMP IP Initialize
 *
 * @return		none
 *********************************************************************************************
 */
void IP_AMP_init(void)
{

#if defined(__AMP)
	CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_AMP, ENABLE);
	/* Setting internal amp */
	TSB_AMP->CTLA = AMP_DISABLE(AMPGAIN_FOLD_2_0);
	TSB_AMP->CTLB = AMP_DISABLE(AMPGAIN_FOLD_2_0);
	TSB_AMP->CTLC = AMP_DISABLE(AMPGAIN_FOLD_2_0);
#endif

}

/**
 * @brief   VE_GetPLSLF
 *
 * @param   _ipdrv  Select the IP table.
 * @return  ve_plslf_state_e  PLSLF flag on  : ve_plslf_state_on
 *                                       off : ve_plslf_state_off
 * @note    VEMCTLF:bit4 status flag register-Minimum disparity of pulse width
 *
 */
ve_plslf_state_e VE_GetPLSLF(const ipdrv_t* const _ipdrv)
{
	TSB_VE_TypeDef* const VEx = _ipdrv->VEx;

	return (
	  ((*((__IO uint32_t*)BITBAND_PERI(&VEx->MCTLF, B_PLSLF))) != 0)?
	  ve_plslf_state_on : ve_plslf_state_off
	);
}
/*********************************** END OF FILE ******************************/
