/**
 ****************************************************************************
 * @file:	 usercon.c
 * @brief:	 User Control
 * @version  V1.1
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>
#include <stdio.h>

#include "ipdefine.h"
#include "mcuip_drv.h"
#include "dac_drv.h"

#define DEFINE_APP
#include "usercon.h"
#undef	DEFINE_APP


/*===================================================================*
	  Macro Definition
 *===================================================================*/

/*===================================================================*
	  Local Constant Data Definition
 *===================================================================*/
/*Initial value*/
#define cININT_VALUE		(0)
#define cININT_FF			(0xff)
#define cAD_ININT_FFF		(0xfff)

/* Key settting */
#define S_SW1				TSB_PG_DATA_PG3										/* slideSW1 or 8(KEY1) */
#define S_SW2				TSB_PG_DATA_PG4										/* slideSW2 or 9(KEY2) */
#define USW1				TSB_PD_DATA_PD1										/* pushSW1 DACSW */

#define	cKEY_CHATA_CNT		(20)												/* [cnt] chattering counter for KEY SW */
#define	cNO_ENTER_KEY		(0)

/* Soft ADC Setting */
#define	cADUNIT_USR			TSB_ADC												/* User ad data ADCUnit */
#define	cADAUNIT_USR		TSB_ADA												/* User ad data ADAUnit */
#define	cADTRG_USR			ADC_TRG_SINGLE										/* ADC Trigger Type */
#define cADCH_ADKEY 		ADC_AIN0											/* ADC Channel for AD Key */
#define cADREG_ADKEY		ADC_REG4											/* Result register No for AD Key */
#define cADCH_VR			ADC_AIN0											/* ADC Channel for VR */
#define cADREG_VR			ADC_REG4											/* Result register No for VR */
#define cADCH_TEMP			ADC_AIN13											/* ADA Channel for TEMP */
#define cADREG_TEMP			ADC_REG7											/* Result register No for TEMP */
#define cADAVECNT			(10)												/* ADC average count */
#define cPUSHSW_CHATA_CNT	(5) 												/* [cnt] Chattering counter for Push SW */


/* AD speed control setting */
#define	cAD_MIN				(0x10)												/* Voltage value 0.3V */
#define	cAD_MAX				(0xF0)												/* Voltage value 4.7V */
#define	cSPEED_USER_MIN		(12) 												/* [Hz] Min Target speed of motor */
#define	cSPEED_USER_MAX		(200) 												/* [Hz] Max Target speed of motor */


/* AD Temp setting */
#define	cONE_ABOVE			(1)


/* DACMODE setting */
#define	cDACMODE_MIN		(0)													/* DACMODE count0 */
#define	cDacMODE_MAX		(3)													/* DACMODE count3 */


/* LED setting */					
#define	LED_FORCED			TSB_PV_DATA_PV0										/* LED4 */
#define	LED_EMG				TSB_PB_DATA_PB6										/* LED6 */
#define	cLED_ON				(1)													/* LED ON level */
#define	cLED_OFF			(0)													/* LED OFF level */


/* flag status */
#define	cFLG_ON				(1)
#define	cFLG_OFF			(0)


/* UART Setting */
#define UART_BRD_INIT		((uint32_t)0x00A6002B)
#define UART_CLK_INIT		((uint32_t)0x00000000)
#define UART_CR0_INIT		((uint32_t)0x00000001)
#define UART_CR1_INIT		((uint32_t)0x00000040)
#define	UART_SEND_WAIT		(1000)
#define	UART_COUNT			(0)
#define c10MHz				(1000000)
#define	cLOW_ACTIVE			(0)
#define	cSWRST10			(0x2U)
#define	cSWRST01			(0x1U)
#define	cSWRSTF				(0x80)


/* User_interface Setting */
#define cROTATION_CW		(1)
#define cROTATION_CCW		(0)
#define	cPSW1_Hi			(1)
#define	cPSW1_Low			(0)
#define	cStrCH0				("CH0")
#define	cINV_MIN_TEMP		(-200)
#define	cINV_MAX_TEMP		(1000)

#define	cSendBufSIZE		(80+1)

/*===================================================================*
	  Typedef Definition
 *===================================================================*/
typedef struct
{
	uint8_t		select;															/* Dac output select */
	uint8_t		motch;															/* Dac motor ch select */
	uint8_t		datsft0;														/* Dac data shift for data0 */
	uint8_t		datsft1;														/* Dac data shift for data1 */
	uint8_t		datsft2;														/* Dac data shift for data2 */
	uint8_t		datsft3;														/* Dac data shift for data3 */
} dac_t;

#if defined(__CONTROL_MOTOR_CH2)
extern	int32_t	target_spd2;
#endif /* __CONTROL_MOTOR_CH2 */

#if defined(__CONTROL_MOTOR_CH1)
extern	int32_t	target_spd1;
#endif /* __CONTROL_MOTOR_CH1 */

#if defined(__CONTROL_MOTOR_CH0)
extern	int32_t	target_spd0;
#endif /* __CONTROL_MOTOR_CH0 */


/*===================================================================*
	  Local Variable Definition
 *===================================================================*/
 
static sw_t		portbuf;
static sw_t		keydata;													/* Key data */
static uint16_t	keyportbuf_old; 											/* Previous Key port data */
static uint8_t		key_chatacnt;												/* Chattering counter for Key */

static sw_t		swdata;														/* Push sw data */
static sw_t		swportbuf_old; 												/* Previous push sw port data */
static uint8_t		sw_chatacnt;												/* Chattering counter for push sw */

static addat_t		ad_vr;														/* AD value of volume register */
static addat_t		ad_inv;														/* AD value of inveter register */
static uint8_t		temp_data_get_flag;


static uint8_t     motor_state;

static uint8_t		stop_output;												/* stop_outputflag */

static uint8_t		rota_dir;													/* Rotation direction setting */

/* No, MotorCh, sft0, sft1, sft2, sft3 */
dac_t 		dac = { 0, 0, 0, 0, 0, 0 };											/* Dac setting. */
static uint8_t 	prev_dacmode;												/* Previous DAC Key */
static uint8_t		send_step;
static uint16_t	send_wait_time;
static uint8_t		log_req_flg;							 					/* log request flag*/

static char		sendbuf[cSendBufSIZE];
static uint8_t		sendstatus;
static uint8_t		sendpoint;

static float32_t	log_temppara;



/*===================================================================*
	  Local Proto Type Definition
 *===================================================================*/
static void init_Uikey(void);
static void Uikeyscan(void);
static void init_soft_adc(void);
static void soft_adc_getdata(void);
static void soft_adc_avecal(addat_t* const _ad_dat);
static int16_t temp_check(uint16_t	temp_ad);
void user_interface(void);
static void init_led(void);
static void led_display(void);

static void init_uart(void);

static void send_data_set(void);


/**
 *********************************************************************************************
 * @brief		init_user_control
 *
 * @return		none
 *********************************************************************************************
 */
void	init_user_control(void)
{

	init_Uikey();																/* Initial setting of key input */
	init_soft_adc();															/* Initial setting of soft adc */
	init_led();																	/* Initial setting of LED output */
	init_uart();																/* Initial setting of UART */
	
	/* Initialize to variable */
	stop_output = cFLG_ON;
	ad_vr.avedat = cAD_ININT_FFF;
	

}

/**
 *********************************************************************************************
 * @brief		user_control
 *
 * @return		none
 *********************************************************************************************
 */
void	user_control(void)
{
	/***** Get AD Data *****/
	soft_adc_getdata();

	/***** key input *****/
	Uikeyscan();

	  
	/***** user_interface *****/
	user_interface();

	/***** led_display *****/
	led_display();
	
	/***** UART send *****/
	send_data_set();
}


/**
 *********************************************************************************************
 * @brief		Initialize key.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_Uikey(void)
{
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTD, ENABLE);
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTG, ENABLE);
	TSB_PD_IE_PD1IE = SET;
	TSB_PG_IE_PG4IE = SET;
	TSB_PG_IE_PG3IE = SET;
}

/**
 *********************************************************************************************
 * @brief		keyscan
 *
 * @return		none
 *********************************************************************************************
 */
static void Uikeyscan(void)
{
  	/* rotation,phease */

	*((uint8_t*)&portbuf) = cININT_FF;											
	portbuf.sw1 = S_SW1;
	portbuf.sw2 = S_SW2;
	

	if (*((uint8_t*)&portbuf) == *((uint8_t*)&keyportbuf_old))
	{
		if (key_chatacnt != cNO_ENTER_KEY)
		{
		  	key_chatacnt--;
			if (key_chatacnt == cNO_ENTER_KEY)
			{
				*((uint8_t*)&keydata) = ~(*((uint8_t*)&portbuf));
			}
		}
	}
	else
	{
		*((uint8_t*)&keyportbuf_old) = *((uint8_t*)&portbuf);
		key_chatacnt = cKEY_CHATA_CNT;
	}

	/* DAC */
	*((uint8_t*)&portbuf) = cININT_VALUE;
	portbuf.sw1 = USW1;

	if (*((uint8_t*)&portbuf) == *((uint8_t*)&swportbuf_old))
	{
		if (sw_chatacnt != cNO_ENTER_KEY)
		{
		  	sw_chatacnt--;
			if (sw_chatacnt == cNO_ENTER_KEY)
			{
				*((uint8_t*)&swdata) = ~*((uint8_t*)&portbuf);
			}
		}
	}
	else
	{
		*((uint8_t*)&swportbuf_old) = *((uint8_t*)&portbuf);
		sw_chatacnt = cKEY_CHATA_CNT;
	}

}

/**
 *********************************************************************************************
 * @brief		Initialize softconvert adc.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_soft_adc(void)
{
	CG_SetAdcClkSupply(cADUNIT_USR, ENABLE);

	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTJ, ENABLE);
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTL, ENABLE);
	CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_ADC_A, ENABLE);
	CG_SetFsysSupplyPeriphB(CG_FSYS_PERIPH_ADC_C, ENABLE);
	
	ADC_SetSWTrg(cADUNIT_USR,cADREG_VR,TRG_ENABLE(cADCH_VR));
	ADC_SetSWTrg(cADAUNIT_USR,cADREG_TEMP,TRG_ENABLE(cADCH_TEMP));

	ADC_Enable(cADUNIT_USR, ADC_CLKMOD_160M, ADC_REFSEL_AIN, ADC_RCUT_NORMAL);
	
	ADC_Start(cADUNIT_USR, cADTRG_USR); 										/* ADCUNIT adc start */
	ADC_Start(cADAUNIT_USR, cADTRG_USR);										/* ADAUNIT adc start */
}


/**
 *********************************************************************************************
 * @brief		soft_adc_getdata
 *
 * @return		none
 *********************************************************************************************
 */
static void soft_adc_getdata(void)
{
	if (ADC_GetConvertState(cADUNIT_USR, cADTRG_USR) == DONE)					/*  ADCUNIT finish adc? */
	{
		ad_vr.rawdat = ADC_GetConvertResult(cADUNIT_USR, cADREG_VR);
		ad_vr.rawdat.Bit.ADResult >>= 4;
		ad_vr.rawdat.Bit.ADResult &= 0xff;
		soft_adc_avecal(&ad_vr);
	
	}

	if (ADC_GetConvertState(cADAUNIT_USR, cADTRG_USR) == DONE)					/*  ADAUNIT finish adc? */
	{
		ad_inv.rawdat = ADC_GetConvertResult(cADAUNIT_USR, cADREG_TEMP);
		soft_adc_avecal(&ad_inv);
		if(ad_inv.avecnt == cININT_VALUE)
		{
			temp_data_get_flag = cFLG_ON;
		}
	}
	ADC_Start(cADUNIT_USR, cADTRG_USR); 										/* ADCUNIT adc start */
	ADC_Start(cADAUNIT_USR, cADTRG_USR);										/* ADAUNIT adc start */
}



/**
 *********************************************************************************************
 * @brief		soft_adc_avecal
 *
 * @param		addat_t* const	_ad_dat
 *
 * @return		none
 *********************************************************************************************
 */
static void soft_adc_avecal(addat_t* const _ad_dat)
{

	if (_ad_dat->avecnt == cININT_VALUE)
	{
		/* first? */
		_ad_dat->maxdat = _ad_dat->rawdat.Bit.ADResult;
		_ad_dat->mindat = _ad_dat->rawdat.Bit.ADResult;
	}
	else
	{
		if (_ad_dat->rawdat.Bit.ADResult > _ad_dat->maxdat)
		{
			_ad_dat->maxdat = _ad_dat->rawdat.Bit.ADResult;
		}
		else if (_ad_dat->rawdat.Bit.ADResult < _ad_dat->mindat)
		{
			_ad_dat->mindat = _ad_dat->rawdat.Bit.ADResult;
		}
	}

	_ad_dat->sumdat += _ad_dat->rawdat.Bit.ADResult;

	_ad_dat->avecnt++;
	if (_ad_dat->avecnt >= cADAVECNT)
	{
		_ad_dat->avecnt = cININT_VALUE;
		_ad_dat->sumdat -= _ad_dat->maxdat;
		_ad_dat->sumdat -= _ad_dat->mindat;
		_ad_dat->avedat = (_ad_dat->sumdat / (cADAVECNT - 2));
		_ad_dat->sumdat = cININT_VALUE;
	}

}

/**
 ********************************************************************************************
 * @brief		Temperature calculation from AD.
 *
 * @param[in]	uint8_t	temp_ad	.
 *
 * @return		none
 ********************************************************************************************
 */
#define		cTEMPAD_MAX		(3656)
#define		cTEMPAD_MIN		(345)

#define		cTEMP_MAX		(1000)
#define		cTEMP_MIN		(-200)

typedef struct {
	int16_t		temp_value;
	uint16_t	ad_value;
} TYPE_TEMP;

TYPE_TEMP	t_temppara[] = {
		{	-200,	3656},
		{	-100,	3323},
		{	   0,	2958},
		{	 100,	2550},
		{	 200,	2146},
		{	 300,	1755},
		{	 400,	1419},
		{	 500,	1128},
		{	 600,	 896},
		{	 700,	 711},
		{	 800,	 565},
		{	 900,	 439},
		{	1000,	 345}
};

static int16_t temp_check(uint16_t temp_ad)
{
	int16_t	percent, slope, def, temp_data;
	uint8_t	i;

	if(temp_ad < cTEMPAD_MIN)
	{
		temp_data = cTEMP_MAX;
	}
	else if(temp_ad > cTEMPAD_MAX)
	{
		temp_data = cTEMP_MIN;
	}
	else
	{
		for(i = 0; i < (sizeof(t_temppara)/sizeof(t_temppara[0])) ; i++)
		{
			if(temp_ad >= t_temppara[i].ad_value )
			{
				slope = (int16_t)(t_temppara[i - cONE_ABOVE].ad_value - t_temppara[i].ad_value);
				def = (int16_t)(t_temppara[i - cONE_ABOVE].ad_value - temp_ad);
				percent = (int16_t)((t_temppara[i].temp_value - t_temppara[i - cONE_ABOVE].temp_value) * def / slope);
				temp_data = (int16_t)(t_temppara[i - cONE_ABOVE].temp_value + percent);
				break;
			}
		}
	}

	return(temp_data);
}



/**
 *********************************************************************************************
 * @brief		user_interface
 *
 * @return		none
 *********************************************************************************************
 */
void user_interface(void)
{
	int32_t motor_speed = cININT_VALUE;
	uint16_t ad_vr_temp = ad_vr.avedat;
 	vector_t* const _motor = &Motor_ch0;	

	/* ADvolme setting */    
	if(ad_vr_temp < cAD_MIN)
	{
		motor_state = cFLG_ON;
		motor_speed = cStop;
	}
	else
	{
		if(motor_state == cFLG_ON)
		{
			if(ad_vr_temp < cAD_MIN)
			{
				motor_state = cSPEED_USER_MIN;
			}
			else if(ad_vr_temp < cAD_MAX)
			  {
				motor_speed = ((cSPEED_USER_MAX - cSPEED_USER_MIN) * (((int32_t)ad_vr_temp)-cAD_MIN) / (cAD_MAX - cAD_MIN)) + cSPEED_USER_MIN;
			  }
			  else
			{
				motor_speed =  cSPEED_USER_MAX;
			}
		}
		else
		{
			motor_speed = cStop;
		}
	}

	
	/* motor rotation setting */
	if(rota_dir != (uint8_t)(keydata.sw1))
	{
		motor_state = cFLG_OFF;
		motor_speed = cStop;
	}
	rota_dir = (uint8_t)(keydata.sw1);

	if(rota_dir == cROTATION_CCW)
	{
		motor_speed = -motor_speed;
	}

	/* modul control */
	if(_motor->usr.com_user.modul != (uint8_t)(keydata.sw2))
	{
		motor_state = cFLG_OFF;
		motor_speed = cStop;
	}
	_motor->usr.com_user.modul = (uint8_t)(keydata.sw2);

	
	target_spd0 = motor_speed;
	
	
	/* DADCMODE  switching*/
	if(prev_dacmode != (uint8_t)(swdata.sw1)) 
	{
		if(swdata.sw1 == cPSW1_Hi)
		{
			log_req_flg = cFLG_ON;
			dac.select++;
			if(dac.select > cDacMODE_MAX)
			{
				dac.select = cDACMODE_MIN;
			}
		}
	}
	prev_dacmode = (uint8_t)(swdata.sw1);
}


/**
 *********************************************************************************************
 * @brief		Initialize LED.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_led(void)
{
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTV, ENABLE);
	TSB_PV_CR_PV0C = SET;
	TSB_PV_CR_PV1C = SET;	
}
 /**
 *********************************************************************************************
 * @brief		led_display
 *
 * @return		none
 *********************************************************************************************
 */
static void led_display(void)
{
	vector_t* const _motor = &Motor_ch0;

 	if((_motor->stage.main >= cInitposition) && (_motor->stage.main < cSteady_A))
 	{
		LED_FORCED	= cLED_ON;
 	}
 	else{
 		LED_FORCED = cLED_OFF;
 	}
}
 

/**
 *********************************************************************************************
 * @brief		Initialize Test point
 *
 * @return		none
 *********************************************************************************************
 */
void init_TP(void)
{
	/* port setting */
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTC, ENABLE);
	TSB_PC->CR |= ((1 << 7U) | (1 << 6U) | (1 << 5U) | (1 << 4U));
}

/**
 *********************************************************************************************
 * @brief		Initialize UART.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_uart(void)
{
	/* port setting(PC0, PC1) */
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTC, ENABLE);
	TSB_PC->FR1 |= ((1 << 0) | (1 << 1U));
	TSB_PC->IE |= (1 << 1U);	/* P0:TXD */
	TSB_PC->CR |= (1 << 0U);	/* P1:RXD */

	/* UART setting(UART0) */
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_UART0, ENABLE);

	TSB_UART0->SWRST = cSWRST10;
	TSB_UART0->SWRST = cSWRST01;

	while(TSB_UART0->SWRST==  cSWRSTF)
	{
	}

	TSB_UART0->BRD = UART_BRD_INIT;
	TSB_UART0->CLK = UART_CLK_INIT;
	TSB_UART0->CR0 = UART_CR0_INIT;
	TSB_UART0->CR1 = UART_CR1_INIT;
	NVIC_EnableIRQ(INTSC0TX_IRQn);

	send_step = cININT_VALUE;
	send_wait_time = cININT_VALUE;
}


/**
 *********************************************************************************************
 * @brief		Transmission completion interrupt.
 *
 * @return		none
 *********************************************************************************************
 */

/* Define for UART	*/
#define UART_TXE_ENABLE		((uint32_t)0x00000002)
#define UART_TXE_DISABLE	((uint32_t)0x00000000)
#define UART_TX_FLAG_FINISH	((uint32_t)0x00004000)
#define UART_TX_FLAG_CLR	((uint32_t)0x00004000)
#define UART_ERR_CHECK		((uint32_t)0x0000001F)
#define UART_ERR_CLR		((uint32_t)0x0000001F)

void INT_uart0tx(void)
{
	TSB_UART0->TRANS = UART_TXE_DISABLE;
	if((TSB_UART0->ERR & UART_ERR_CHECK) == 0)
	{
		sendpoint++;
		if(sendpoint >= strlen(sendbuf))
		{
			sendpoint = 0;
			sendstatus = cFLG_OFF;
			TSB_UART0->SR = UART_TX_FLAG_CLR;
		}
		else
		{
			TSB_UART0->DR = sendbuf[sendpoint];
			TSB_UART0->TRANS = UART_TXE_ENABLE;
		}
	}
	else
	{
		TSB_UART0->ERR = UART_ERR_CLR;
		TSB_UART0->SR = UART_TX_FLAG_CLR;
		sendstatus = cFLG_OFF;
	}
}


const char*	strDacMode[] = {
	"DAC Data = A:TMPREG0 B:TMPREG1 C:TMPREG2 D:theta.half[1]\n",
	"DAC Data = A:Id_ref B:Id C:Iq_ref D:Iq\n",
	"DAC Data = A:omega_com.half[1] B:omega.half[1] C:omega_def D:Iq_ref\n",
	"DAC Data = A:TMPREG0 B:Iq_ref C:Id_ref D:omega.half[1]\n"
};


/**
 *********************************************************************************************
 * @brief		UART Send data setting.
 *
 * @return		none
 *********************************************************************************************
 */
static void send_data_set(void)
{
	TSB_VE_TypeDef* pVE;
	float32_t	VE_value,VE_value_U,VE_value_V,VE_value_W;
	uint32_t	set_value;
	vector_t* const _motor = &Motor_ch0;
	int16_t	temp_data;
	
	if(send_wait_time !=UART_COUNT)								
	{
		send_wait_time--;												
	}
	
	if(sendstatus == cFLG_OFF)
	{
		memset(sendbuf, 0, sizeof(sendbuf));
		pVE = TSB_VE0;
		switch(send_step)
		{
			case 0:																/* Initial Status */
				sprintf(sendbuf,"Initial Status\n");
				send_step++;
				break;
	
			case 1:																/* Control CH0 */
				sprintf(sendbuf,"Control CH = %s\n",cStrCH0) ;
				send_step++;
				break;
	
			case 2:																/* Carrier Frequency */
				set_value = (uint32_t)(c10MHz / cPWMPRD_CH0);
				sprintf(sendbuf,"Carrier Frequency = %dHz\n",set_value);
				send_step++;
				break;
	
			case 3:																/* Dead Time */
				sprintf(sendbuf,"DEAD TIME = %3.1fus\n",cDEADTIME_CH0);
				send_step++;
				break;
	
			case 4:																/* Gate Active H/H or L/L */		
#if (cPOLH_CH0 == cLOW_ACTIVE)
				sprintf(sendbuf,"Gate Active = L/L\n");
#else
				sprintf(sendbuf,"Gate Active = H/H\n");
#endif
				send_step++;
				break;
	
			case 5:																/* Position Detect 3Shunt */
				sprintf(sendbuf,"Position Detect = 3Shunt\n");
				send_step++;
				break;
				
			case 6:																/* VDC Voltage */
				VE_value = (float32_t)(_motor->drv.Vdc * cV_MAX_CH0 / FIX_15);
				sprintf(sendbuf,"VDC Voltage = %4.1fV\n",VE_value);
				send_step++;
				break;				
	
			case 7:																/* Inverter Temp */
				if(temp_data_get_flag == cFLG_OFF) 
				{
					break;
				}

				temp_data = temp_check(ad_inv.avedat);
				if((cINV_MIN_TEMP <= temp_data) && (temp_data <= cINV_MAX_TEMP))
				{
				  	log_temppara = (float32_t)temp_data / 10;
					sprintf(sendbuf,"Inverter Temp = %0.1f degree\n",log_temppara);
				}
				else
				{
					sprintf(sendbuf,"Inverter Temp = Error\n");
				}
				send_step++;
				break;

			case 8:																/* UVW 0A Voltage */
				VE_value_U = (float32_t)(pVE->IAADC * 5) / 65535;
				
				VE_value_V = (float32_t)(pVE->IBADC * 5) / 65535;
				
				VE_value_W = (float32_t)(pVE->ICADC * 5) / 65535;
				
			  	sprintf(sendbuf,"UVW 0A Voltage = %4.2fV/%4.2fV/%4.2fV\n",VE_value_U,VE_value_V,VE_value_W);
				
			  	send_step++;
				break;
	
			case 9:																/* DAC Data */
				if(dac.select <= cDacMODE_MAX) 
				{
					sprintf(sendbuf,"%s", strDacMode[dac.select]);
				  	send_step++;
				}
				break;
	
			case 10:															/* Internal AMP */
				sprintf(sendbuf,"Internal AMP = No\n");
				send_step++;
				break;
	
			case 11:															/* Direction */
				if(rota_dir == cROTATION_CW)
				{
					sprintf(sendbuf,"Direction = CW\n");
				}
				else
				{
					sprintf(sendbuf,"Direction = CCW\n");
				}
				send_step++;
				break;
	
			case 12:															/* Modulation */
				if(_motor->usr.com_user.modul == c2PHASE)
				{
					sprintf(sendbuf,"Modulation = 2Phase\n");
				}
				else
				{
					sprintf(sendbuf,"Modulation = 3Phase\n");
				}
				send_step++;
				break;

			case 13:															/* Key Operation */
				sprintf(sendbuf,"Key Operation = Volume SW\n");
				send_wait_time = UART_SEND_WAIT;
				send_step++;
				break;				
				
			default:
				if(log_req_flg == cFLG_ON)											
				{
					log_req_flg = cFLG_OFF;											
					if(dac.select <= cDacMODE_MAX)
					{
						sprintf(sendbuf,"%s", strDacMode[dac.select]);
					}
					break;
				}
				else
				{
					if((_motor->stage.main == cEmergency) || (target_spd0 == cStop))
					{
						if(stop_output == cFLG_OFF)
						{
							if(target_spd0 == cStop)
							{
								sprintf(sendbuf,"0Hz\n");
							}
							else{
								sprintf(sendbuf,"EMG_S\n");
							}
							stop_output = cFLG_ON;
						}
					}
					else
					{
						if(send_wait_time == UART_COUNT)
						{
							sprintf(sendbuf,"%dHz\n",target_spd0);
							stop_output = cFLG_OFF;
							send_wait_time = UART_SEND_WAIT;
						}
					}
				}
		}
		
		if(strlen(sendbuf))
		{
			sendstatus=cFLG_ON;
			sendpoint=0;
			TSB_UART0->DR = sendbuf[sendpoint];									/* Set data to UART send buffer	*/
			TSB_UART0->TRANS = UART_TXE_ENABLE;									/* Set send enable				*/
		}
	}
}


/**
 ********************************************************************************************
 * @brief		Initialize NBD.
 *
 * @return		none
 ********************************************************************************************
 */
void init_NBD(void)
{
	/* Port */
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTF, ENABLE);
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTN, ENABLE);

	/* NBDSYNC, NBDCLK, NBDDATA0,1 (PF4-7) */
	TSB_PF->FR3 |= ((1 << 7U) | (1 << 6U) | (1 << 5U) | (1 << 4U));
	TSB_PF->CR |= ((1 << 7U) | (1 << 6U));
	TSB_PF->IE |= ((1 << 7U) | (1 << 6U) | (1 << 5U) | (1 << 4U));

	/* NBDDATA2,3 (PN0-1) */
	TSB_PN->FR3 |= ((1 << 1U) | (1 << 0U));
	TSB_PN->CR |= ((1 << 1U) | (1 << 0U));
	TSB_PN->IE |= ((1 << 1U) | (1 << 0U));

	/* Initialize */
	TSB_NBD->CR1 = 0xf9;
	TSB_NBD->CR0 |= 0x01;
}

/**
 ********************************************************************************************
 * @brief		UiOutDataStart by VE.
 *
 * @param[in]	uint8_t	_intch	.
 *
 * @return		none
 ********************************************************************************************
 */
void UiOutDataStartByVE(uint8_t	_intch)
{
#if defined(__USE_VE_CH0) || defined(__USE_VE_CH1) || defined(__USE_VE_CH2)
	dac_t* const pdac = &dac;

	if (_intch == pdac->motch)		/* Dac output ch = call intve ch? */
	{
		uint16_t* const pdacout = DacData;
		vector_t* pmotor = 0;
		TSB_VE_TypeDef* pVE = 0;

		if (pdac->motch == 0)
		{
		#if defined(__CONTROL_MOTOR_CH0)
			pmotor = &Motor_ch0;
			pVE = Motor_ch0_IP.VEx;
		#endif
		}
		else if (pdac->motch == 1)
		{
		#if defined(__CONTROL_MOTOR_CH1)
			pmotor = &Motor_ch1;
			pVE = Motor_ch1_IP.VEx;
			pVE = TSB_VE1;
		#endif
		}
		else
		{
		#if defined(__CONTROL_MOTOR_CH2)
			pmotor = &Motor_ch2;
			pVE = Motor_ch2_IP.VEx;
			pVE = TSB_VE2;
		#endif
		}

		switch (pdac->select)
		{
			case 0:
				*(pdacout + 0) = (pVE->IA						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 1) = (pVE->IB						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 2) = (pVE->IC						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 3) = pmotor->drv.theta.half[1];
				break;
			case 1:
				*(pdacout + 0) = (pmotor->drv.Id_ref << pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Id	 << pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Iq_ref << pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq	 << pdac->datsft0) ^ 0x8000;
				break;
			case 2:
				*(pdacout + 0) = (pmotor->drv.omega_com.half[1] << pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.omega.half[1] 	<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.omega_dev 		<< pdac->datsft1) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq_ref			<< pdac->datsft2) ^ 0x8000;
				break;
			case 3:
				*(pdacout + 0) = (pVE->IA						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Iq_ref        << pdac->datsft1)        ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Id_ref        << pdac->datsft1)        ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.omega.half[1] << pdac->datsft2)        ^ 0x8000;
				break;
			default:
				*(pdacout + 0) = *(pdacout + 0) + 0x0100;
				*(pdacout + 1) = *(pdacout + 0);
				*(pdacout + 2) = *(pdacout + 0);
				*(pdacout + 3) = *(pdacout + 0);
				break;
		}

		DacWriteStart(DAC_SIO_CH);

	}
#endif /* defined(__USE_VE_CH0) || defined(__USE_VE_CH1) || defined(__USE_VE_CH2) */
}

/**
 *********************************************************************************************
 * @brief		UiOutDataStart for Software Vector Control.
 *
 * @param		uint8_t	_intch	call intve ch
 *
 * @return		none
 *********************************************************************************************
 */
void UiOutDataStart(uint8_t	_intch)
{
#if (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0))
	dac_t* const pdac = &dac;

	if (_intch == pdac->motch)		/* Dac output ch = call intve ch? */
	{
		uint16_t* const pdacout = DacData;
		vector_t* pmotor = 0;

		if (pdac->motch == 0)
		{
		#if defined(__CONTROL_MOTOR_CH0)
			pmotor = &Motor_ch0;
		#endif
		}
		else if (pdac->motch == 1)
		{
		#if defined(__CONTROL_MOTOR_CH1)
			pmotor = &Motor_ch1;
		#endif
		}
		else{
		#if defined(__CONTROL_MOTOR_CH2)
			pmotor = &Motor_ch2;
		#endif
		}

		switch (pdac->select)
		{
			case 0:
				*(pdacout + 0) = (pmotor->drv.Ia				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Ia_raw			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Ic				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = pmotor->drv.theta.half[1];
                                 break;
			case 1:
				*(pdacout + 0) = (pmotor->drv.Id_ref			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Id				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Iq_ref			<< pdac->datsft0) ^ 0x8000;                           
				*(pdacout + 3) = (pmotor->drv.Iq				<< pdac->datsft0) ^ 0x8000;
				break;
			case 2:
				*(pdacout + 0) = (pmotor->drv.omega_com.half[1]	<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.omega.half[1] 	<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.omega_dev 		<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq_ref			<< pdac->datsft0) ^ 0x8000;
				break;
			case 3:
				*(pdacout + 0) = (pmotor->drv.Ia				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Iq_ref			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Id_ref			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.omega.half[1]		<< pdac->datsft2) ^ 0x8000;
				break;

			default:
				*(pdacout + 0) += 0x0100;
				*(pdacout + 1) += 0x0200;
				*(pdacout + 2) += 0x0300;
				*(pdacout + 3) += 0x0400;
				break;
		}

		DacWriteStart(DAC_SIO_CH);

	}
#endif /* (defined(__CONTROL_MOTOR_CH2) && !defined(__USE_VE_CH2)) || (defined(__CONTROL_MOTOR_CH1) && !defined(__USE_VE_CH1)) || (defined(__CONTROL_MOTOR_CH0) && !defined(__USE_VE_CH0)) */
}


/******************************** END OF FILE *********************************/
