/**
 ****************************************************************************
 * @file:	 v7z_board.c
 * @brief:	 Processing dedicated to V7Z board
 * @version  V1.1
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */

#include <stdlib.h>
#include <stdio.h>
#include "ipdefine.h"
#include "mcuip_drv.h"
#include "v7z_board.h"

/*===================================================================*
      Port Definition
 *===================================================================*/

/* RELAY Port Setting */
#define PORT_RELAY_2_OFF		TSB_PV_DATA_PV0 = 0
#define PORT_RELAY_2_ON			TSB_PV_DATA_PV0 = 1

#define PORT_RELAY_OFF			TSB_PA_DATA_PA3 = 0
#define PORT_RELAY_ON			TSB_PA_DATA_PA3 = 1


/* EN_2 Port Setting */
#define PORT_EN_2_OFF			TSB_PV_DATA_PV1 = 0
#define PORT_EN_2_ON			TSB_PV_DATA_PV1 = 1

/* EN_1 Port Setting */
#define PORT_EN_1_OFF			TSB_PA_DATA_PA1 = 0
#define PORT_EN_1_ON			TSB_PA_DATA_PA1 = 1

/* EN_0 Port Setting */
#define PORT_EN_0_OFF			TSB_PA_DATA_PA0 = 0
#define PORT_EN_0_ON			TSB_PA_DATA_PA0 = 1

#define CLEAR_WDT				SIWDT_WriteClearCode

/*===================================================================*
	  Select VDC Check Channel
 *===================================================================*/
#if defined(__CONTROL_MOTOR_CH0)

	#define __VDC_CHECK_CH 	(0)		/* VDC Ch0 */

#elif defined(__CONTROL_MOTOR_CH1)

	#define __VDC_CHECK_CH 	(1)		/* VDC Ch1 */
	
#elif defined(__CONTROL_MOTOR_CH2)

	#define __VDC_CHECK_CH 	(2)		/* VDC Ch2 */

#else
	#error "__VDC_CHECK_CH define error."
#endif

/*===================================================================*
	  Local Constant Data Definition
 *===================================================================*/
#define cV_RelayON			(141.0f * 0.9f)		/* [V] Relay On Voltage */
#define cRelayLen			(3.0f)				/* [s] Time of Voltage check */
#define cRelayLim			(cRelayLen * 2)		/* [s] Time of limitation */

/*===================================================================*
	  Local Variable Definition
 *===================================================================*/
#if defined(__RELAY)
static uint8_t relay_state;
#endif /* defined(__RELAY) */

/*===================================================================*
	Function
 *===================================================================*/
/**
 *********************************************************************************************
 * @brief		Initial setting of Relay Port.
 *
 * @return		none
 *********************************************************************************************
 */
void init_Relay(void)
{
#if defined(__RELAY)
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTV, ENABLE);
	TSB_PV_DATA_PV0 = 0;
	TSB_PV_CR_PV0C = 1;

	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTA, ENABLE);
	TSB_PA_DATA_PA3 = 0;
	TSB_PA_CR_PA3C = 1;
#endif /* defined(__RELAY) */
}

/**
 *********************************************************************************************
 * @brief		Turn on Relay.
 *
 * @return		none
 *********************************************************************************************
 */
void Relay_On(void)
{
#if defined(__RELAY)
	uint32_t len_cnt;
	uint32_t lim_cnt;

	CLEAR_WDT();

	PORT_RELAY_OFF;
	PORT_RELAY_2_OFF;
	relay_state = OFF;

	len_cnt = 0;
	lim_cnt = 0;

	while ((relay_state != ON) && !(lim_cnt > (cRelayLim / cMAINLOOP_PRD)))
	{
		if (M_Main_Counter >= cMAINLOOP_CNT)
		{
			M_Main_Counter = 0;

			CLEAR_WDT();

			len_cnt++;
			lim_cnt++;
			
#if (__VDC_CHECK_CH == 0)
			if (Motor_ch0.drv.Vdc > (q15_t)(FIX_15 * cV_RelayON / cV_MAX_CH0))
#elif (__VDC_CHECK_CH == 1)
			if (Motor_ch1.drv.Vdc > (q15_t)(FIX_15 * cV_RelayON / cV_MAX_CH1))
#elif (__VDC_CHECK_CH == 2)
			if (Motor_ch2.drv.Vdc > (q15_t)(FIX_15 * cV_RelayON / cV_MAX_CH2))
#endif
			{
				if (len_cnt > (cRelayLen / cMAINLOOP_PRD))
				{
					relay_state = ON;
				}
			}
			else
			{
				len_cnt = 0;
			}
		}
	}
	if (relay_state == ON)
	{
	  	PORT_RELAY_ON;
	  	PORT_RELAY_2_ON;
	}
	else
	{
		/* relay on error */
	}
#endif /* defined(__RELAY) */
}

/**
 *********************************************************************************************
 * @brief		Initial Setting of PreDiver EN_x.
 *
 * @return		none
 *********************************************************************************************
 */
void init_PreDrvEn(void)
{
#if defined(__CONTROL_MOTOR_CH2)
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTV, ENABLE);
	TSB_PV_DATA_PV1 = 0;
	TSB_PV_CR_PV1C = 1;
#endif /* __CONTROL_MOTOR_CH2 */

#if defined(__CONTROL_MOTOR_CH1)
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTA, ENABLE);
	TSB_PA_DATA_PA1 = 0;
	TSB_PA_CR_PA1C = 1;
#endif /* __CONTROL_MOTOR_CH1 */

#if defined(__CONTROL_MOTOR_CH0)
	CG_SetFsysSupplyPeriphA(CG_FSYS_PERIPH_PORTA, ENABLE);
	TSB_PA_DATA_PA0 = 0;
	TSB_PA_CR_PA0C = 1;
#endif /* __CONTROL_MOTOR_CH0 */
}

/**
 *********************************************************************************************
 * @brief		PreDiver EN_0 Control.
 *
 * @return		none
 *********************************************************************************************
 */
void PreDrvEn0_Control(void)
{
	#if defined(__CONTROL_MOTOR_CH0)

	switch (Motor_ch0.stage.main)
	{
		case cStop:
			PORT_EN_0_ON;
			break;
		case cEmergency:
			PORT_EN_0_OFF;
			break;
		default:
			break;
	}
	
	#endif /* __CONTROL_MOTOR_CH0 */
}

/**
 *********************************************************************************************
 * @brief		PreDiver EN_1 Control.
 *
 * @return		none
 *********************************************************************************************
 */
void PreDrvEn1_Control(void)
{
	#if defined(__CONTROL_MOTOR_CH1)

	switch (Motor_ch1.stage.main)
	{
		case cStop:
			PORT_EN_1_ON;
			break;
		case cEmergency:
			PORT_EN_1_OFF;
			break;
		default:
			break;
	}

	#endif /* __CONTROL_MOTOR_CH1 */
}

/**
 *********************************************************************************************
 * @brief		PreDiver EN_2 Control.
 *
 * @return		none
 *********************************************************************************************
 */
void PreDrvEn2_Control(void)
{
	#if defined(__CONTROL_MOTOR_CH2)

	switch (Motor_ch2.stage.main)
	{
		case cStop:
			PORT_EN_2_ON;
			break;
		case cEmergency:
			PORT_EN_2_OFF;
			break;
		default:
			break;
	}

	#endif /* __CONTROL_MOTOR_CH2 */
}

/******************************** END OF FILE *********************************/
